#
# Chris Lumens <clumens@redhat.com>
# Peter Jones <pjones@redhat.com>
#
# Copyright 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.errors import KickstartParseWarning
from pykickstart.version import versionToLongString, FC6, F24, F34
from pykickstart.base import BaseData, DeprecatedCommand, KickstartCommand, RemovedCommand
from pykickstart.options import KSOptionParser

import warnings
from pykickstart.i18n import _

class FC6_DmRaidData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)

        self.name = kwargs.get("name", "")
        self.devices = kwargs.get("devices", [])
        self.dmset = kwargs.get("dmset", None)

    def __eq__(self, y):
        if not y:
            return False

        return self.name == y.name and self.devices == y.devices

    def __ne__(self, y):
        return not self == y

    def __str__(self):
        retval = BaseData.__str__(self)
        retval += "dmraid --name=%s" % self.name

        for dev in self.devices:
            retval += " --dev=\"%s\"" % dev

        return retval + "\n"

class FC6_DmRaid(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=60, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.dmraids = kwargs.get("dmraids", [])

    def __str__(self):
        retval = ""
        for dm in self.dmraids:
            retval += dm.__str__()

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="dmraid", description="define a software RAID device", version=FC6)
        op.add_argument("--name", required=True, version=FC6, help="Name of dmraid")
        op.add_argument("--dev", dest="devices", action="append",
                        required=True, version=FC6, help="device to add to the dmraid")
        return op

    def parse(self, args):
        dm = self.dataClass()   # pylint: disable=not-callable
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_obj(ns, dm)
        dm.name = dm.name.split('/')[-1]
        dm.lineno = self.lineno

        # Check for duplicates in the data list.
        if dm in self.dataList():
            warnings.warn(_("A DM RAID device with the name %(name)s and devices %(devices)s has already been defined.") % {"name": dm.name, "devices": dm.devices}, KickstartParseWarning)

        return dm

    def dataList(self):
        return self.dmraids

    @property
    def dataClass(self):
        return self.handler.DmRaidData

class F24_DmRaid(DeprecatedCommand, FC6_DmRaid):
    def __init__(self):  # pylint: disable=super-init-not-called
        DeprecatedCommand.__init__(self)

    def _getParser(self):
        op = FC6_DmRaid._getParser(self)
        op.description += "\n\n.. deprecated:: %s" % versionToLongString(F24)
        return op

class F34_DmRaid(RemovedCommand, F24_DmRaid):
    def _getParser(self):
        op = F24_DmRaid._getParser(self)
        op.description += "\n\n.. versionremoved:: %s" % versionToLongString(F34)
        return op



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import unittest
from tests.baseclass import CommandTest
from pykickstart.errors import KickstartParseError, KickstartDeprecationWarning
from pykickstart.commands.timezone import FC3_Timezone, FC6_Timezone, F18_Timezone, F25_Timezone, RHEL7_Timezone


class Timezone_TestCase(unittest.TestCase):
    def runTest(self):
        for cmd_class in [FC6_Timezone, F25_Timezone, RHEL7_Timezone]:
            cmd = cmd_class()
            op = cmd._getParser()
            for action in op._actions:
                if '--isUtc' in action.option_strings:
                    self.assertFalse(action.default)

        cmd = F18_Timezone()
        self.assertEqual(cmd.__str__(), '')


class FC3_TestCase(CommandTest):
    command = "timezone"

    def runTest(self):
        # assert defaults
        self.assertFalse(FC3_Timezone().isUtc)
        self.assertFalse(F18_Timezone().nontp)

        # pass
        self.assert_parse("timezone Eastern", "timezone  Eastern\n")

        # On FC6 and later, we write out --isUtc regardless of what the input was.
        if self.__class__.__name__ == "FC3_TestCase":
            self.assert_parse("timezone --utc Eastern", "timezone --utc Eastern\n")
        else:
            self.assert_parse("timezone --utc Eastern", "timezone --isUtc Eastern\n")

        # fail
        self.assert_parse_error("timezone")
        self.assert_parse_error("timezone Eastern Central")
        self.assert_parse_error("timezone --blah Eastern")
        self.assert_parse_error("timezone --utc")
        self.assert_parse_error("timezone --bogus-option")

        # extra test coverage
        cmd = self.handler().commands[self.command]
        cmd.timezone = None
        self.assertEqual(cmd.__str__(), "")

class FC6_TestCase(FC3_TestCase):
    def runTest(self):
        FC3_TestCase.runTest(self)

        # pass
        self.assert_parse("timezone --isUtc Eastern", "timezone --isUtc Eastern\n")

        # fail
        self.assert_parse_error("timezone --isUtc")

class F18_TestCase(FC6_TestCase):
    def runTest(self):
        # pass
        self.assert_parse("timezone --utc Europe/Prague")
        self.assert_parse("timezone --isUtc Europe/Prague\n")
        self.assert_parse("timezone --isUtc Eastern", "timezone Eastern --isUtc\n")
        self.assert_parse("timezone Europe/Prague")
        self.assert_parse("timezone Europe/Prague --nontp",
                          "timezone Europe/Prague --nontp\n")
        self.assert_parse("timezone Europe/Prague "
                          "--ntpservers=ntp.cesnet.cz,tik.nic.cz")
        self.assert_parse("timezone Europe/Prague --ntpservers=ntp.cesnet.cz",
                          "timezone Europe/Prague --ntpservers=ntp.cesnet.cz\n")

        # fail
        self.assert_parse_error("timezone")
        self.assert_parse_error("timezone Eastern Central")
        self.assert_parse_error("timezone --blah Eastern")
        self.assert_parse_error("timezone --utc")
        self.assert_parse_error("timezone --isUtc")
        self.assert_parse_error("timezone Europe/Prague --nontp "
                                "--ntpservers=ntp.cesnet.cz")
        self.assert_parse_error("timezone Europe/Prague --ntpservers="
                                "ntp.cesnet.cz, tik.nic.cz")

class F23_TestCase(F18_TestCase):
    def runTest(self):
        # should keep multiple instances of the same URL
        self.assert_parse("timezone --utc Europe/Prague --ntpservers=ntp.cesnet.cz,0.fedora.pool.ntp.org," +
                          "0.fedora.pool.ntp.org,0.fedora.pool.ntp.org,0.fedora.pool.ntp.org",
                          "timezone Europe/Prague --isUtc --ntpservers=ntp.cesnet.cz,0.fedora.pool.ntp.org," +
                          "0.fedora.pool.ntp.org,0.fedora.pool.ntp.org,0.fedora.pool.ntp.org\n")
        self.assert_parse("timezone --utc Europe/Sofia --ntpservers=,0.fedora.pool.ntp.org,")

        # fail
        self.assert_parse_error("timezone Europe/Sofia --nontp --ntpservers=0.fedora.pool.ntp.org,1.fedora.pool.ntp.org")


class RHEL7_TestCase(F18_TestCase):
    def runTest(self):
        # since RHEL7 command version the timezone command can be used
        # without a timezone specification
        self.assert_parse("timezone --utc")
        self.assert_parse("timezone Europe/Sofia")
        self.assert_parse("timezone --isUtc")
        self.assert_parse("timezone --ntpservers=ntp.cesnet.cz")
        self.assert_parse("timezone --ntpservers=ntp.cesnet.cz,tik.nic.cz")
        # unknown argument
        self.assert_parse_error("timezone --blah")
        # more than two timezone specs
        self.assert_parse_error("timezone foo bar", KickstartParseError, 'One or zero arguments are expected for the timezone command')
        self.assert_parse_error("timezone --utc foo bar", exception=KickstartParseError)
        # just "timezone" without any arguments is also wrong as it really dosn't make sense
        self.assert_parse_error("timezone", KickstartParseError, 'At least one option and/or an argument are expected for the timezone command')

        # fail
        self.assert_parse_error("timezone Europe/Sofia --nontp --ntpservers=0.fedora.pool.ntp.org,1.fedora.pool.ntp.org")


class F25_TestCase(F23_TestCase):
    def runTest(self):
        # since RHEL7 command version the timezone command can be used
        # without a timezone specification
        self.assert_parse("timezone --utc", "timezone --isUtc\n")
        self.assert_parse("timezone --isUtc", "timezone --isUtc\n")
        self.assert_parse("timezone --ntpservers=ntp.cesnet.cz", "timezone --ntpservers=ntp.cesnet.cz\n")
        self.assert_parse("timezone --ntpservers=ntp.cesnet.cz,tik.nic.cz", "timezone --ntpservers=ntp.cesnet.cz,tik.nic.cz\n")
        # normal usage
        self.assert_parse("timezone Europe/Oslo --nontp", "timezone Europe/Oslo --nontp\n")
        # unknown argument
        self.assert_parse_error("timezone --blah")
        # more than two timezone specs
        self.assert_parse_error("timezone foo bar", KickstartParseError, 'One or zero arguments are expected for the timezone command')
        self.assert_parse_error("timezone --utc foo bar", exception=KickstartParseError)
        # just "timezone" without any arguments is also wrong as it really dosn't make sense
        self.assert_parse_error("timezone", KickstartParseError, 'At least one option and/or an argument are expected for the timezone command')

        # fail
        self.assert_parse_error("timezone Europe/Sofia --nontp --ntpservers=0.fedora.pool.ntp.org,1.fedora.pool.ntp.org")


class F32_TestCase(F25_TestCase):
    command = "timezone"

    def runTest(self):
        # Failures
        # unknown argument
        self.assert_parse_error("timezone --blah")
        # more than two timezone specs
        self.assert_parse_error("timezone foo bar", KickstartParseError, 'One or zero arguments are expected for the timezone command')
        self.assert_parse_error("timezone --utc foo bar", exception=KickstartParseError)
        # no options
        self.assert_parse_error("timezone", KickstartParseError, 'At least one option and/or an argument are expected for the timezone command')
        # contradictory options
        self.assert_parse_error("timezone Europe/Sofia --nontp --ntpservers=0.fedora.pool.ntp.org,1.fedora.pool.ntp.org")

        # Successes
        # normal contents
        self.assert_parse("timezone Europe/Prague --ntpservers=ntp.cesnet.cz",
                          "timezone Europe/Prague --ntpservers=ntp.cesnet.cz\n")
        # no timezone spec
        self.assert_parse("timezone --ntpservers=ntp.cesnet.cz",
                          "timezone --ntpservers=ntp.cesnet.cz\n")
        # no ntp wanted
        self.assert_parse("timezone Europe/Oslo --nontp",
                          "timezone Europe/Oslo --nontp\n")

        # New in F32: any variant of UTC should be returned as --utc (again - was in FC3)
        # --utc should result in no warnings
        self.assert_parse("timezone --utc Europe/Bratislava",
                          "timezone Europe/Bratislava --utc\n")
        # but --isUtc should now give warning
        with self.assertWarns(KickstartDeprecationWarning):
            self.assert_parse("timezone --isUtc Europe/Bratislava")


class F33_TestCase(F32_TestCase):
    command = "timezone"

    def runTest(self):
        F32_TestCase.runTest(self)
        # As of Fedora 33 the --ntpservers and --nontp options are considered deprecated.

        # Check using --ntpservers returns appropriate deprecation warning
        with self.assertWarns(KickstartDeprecationWarning):
            self.assert_parse("timezone --ntpservers foo,bar,baz")

        # Check using --ntpservers returns appropriate deprecation warning
        with self.assertWarns(KickstartDeprecationWarning):
            self.assert_parse("timezone --nontp")


if __name__ == "__main__":
    unittest.main()



#
# Martin Gracik <mgracik@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from tests.baseclass import CommandTest
from pykickstart.commands.harddrive import FC3_HardDrive, F33_HardDrive


def attribute_test(test_class, data1, data2, compare_attrs):
    # additional test coverage
    test_class.assertEqual(data1.__str__(), '')

    # test that new objects are always equal
    test_class.assertEqual(data1, data2)
    test_class.assertNotEqual(data1, None)

    # test for objects difference
    for atr in compare_attrs:
        setattr(data1, atr, '')
        setattr(data2, atr, 'test')
        # objects that differ in only one attribute
        # are not equal
        test_class.assertNotEqual(data1, data2)
        test_class.assertNotEqual(data2, data1)
        setattr(data1, atr, '')
        setattr(data2, atr, '')


class HardDrive_TestCase(unittest.TestCase):
    def runTest(self):
        data1 = FC3_HardDrive()
        data2 = FC3_HardDrive()

        attribute_test(self,
                       data1, data2,
                       ['biospart', 'partition', 'dir'])


class FC3_TestCase(CommandTest):
    def runTest(self):
        # pass
        self.assert_parse("harddrive --dir=/install --biospart=part", "harddrive --dir=/install --biospart=part\n")
        self.assert_parse("harddrive --dir=/install --partition=part", "harddrive --dir=/install --partition=part\n")

        self.assertFalse(self.assert_parse("harddrive --dir=/install --partition=sda1") is None)
        self.assertTrue(self.assert_parse("harddrive --dir=/install --partition=sda1") !=
                        self.assert_parse("harddrive --dir=/install --partition=sda2"))
        self.assertFalse(self.assert_parse("harddrive --dir=/install --biospart=80p1") ==
                         self.assert_parse("harddrive --dir=/install --biospart=80p2"))
        self.assertFalse(self.assert_parse("harddrive --dir=/install --biospart=sda1") ==
                         self.assert_parse("harddrive --dir=/other-install --biospart=sda1"))

        # fail
        # required option --dir missing
        self.assert_parse_error("harddrive")
        # required --dir argument missing
        self.assert_parse_error("harddrive --dir")
        # missing --biospart or --partition option
        self.assert_parse_error("harddrive --dir=/install")
        # both --biospart and --partition specified
        self.assert_parse_error("harddrive --dir=/install --biospart=bios --partition=part")
        # --biospart and --partition require argument
        self.assert_parse_error("harddrive --dir=/install --biospart")
        self.assert_parse_error("harddrive --dir=/install --partition")
        # unknown option
        self.assert_parse_error("harddrive --unknown=value")


class F33HardDrive_TestCase(unittest.TestCase):
    def runTest(self):
        data1 = F33_HardDrive()
        data2 = F33_HardDrive()

        attribute_test(self,
                       data1, data2,
                       ['partition', 'dir'])


class F33_TestCase(CommandTest):
    def runTest(self):
        # pass
        self.assert_parse("harddrive --dir=/install --partition=part", "harddrive --dir=/install --partition=part\n")
        self.assertFalse(self.assert_parse("harddrive --dir=/install --partition=sda1") is None)
        self.assertTrue(self.assert_parse("harddrive --dir=/install --partition=sda1") !=
                        self.assert_parse("harddrive --dir=/install --partition=sda2"))

        # fail
        # Ensure these options have been removed.
        self.assert_removed("harddrive", "--biospart")
        # required option --dir missing
        self.assert_parse_error("harddrive")
        # required --dir argument missing
        self.assert_parse_error("harddrive --dir")
        # missing --partition option
        self.assert_parse_error("harddrive --dir=/install")
        # missing --dir option
        self.assert_parse_error("harddrive --partition=sda1")
        # unknown option
        self.assert_parse_error("harddrive --unknown=value")

if __name__ == "__main__":
    unittest.main()



# Andy Lindeberg <alindebe@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest

from pykickstart.base import DeprecatedCommand, RemovedCommand
from tests.baseclass import CommandTest

class FC3_TestCase(CommandTest):
    command = "deviceprobe"

    def runTest(self):
        # pass
        self.assert_parse("deviceprobe")
        self.assert_parse("deviceprobe --cheese", "deviceprobe --cheese\n")
        self.assert_parse("deviceprobe --cracker=CRUNCHY", "deviceprobe --cracker=CRUNCHY\n")

class F29_TestCase(FC3_TestCase):
    def runTest(self):
        # make sure we've been deprecated
        parser = self.getParser("deviceprobe")
        self.assertTrue(issubclass(parser.__class__, DeprecatedCommand))

        # make sure we are still able to parse
        self.assert_parse("deviceprobe")

class RHEL8_TestCase(F29_TestCase):
    pass

class F34_TestCase(F29_TestCase):
    def runTest(self):
        # make sure that deviceprobe is removed
        cmd = self.handler().commands["deviceprobe"]
        self.assertTrue(issubclass(cmd.__class__, RemovedCommand))

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import versionToLongString, FC3, F24, F34
from pykickstart.base import BaseData, DeprecatedCommand, KickstartCommand, RemovedCommand
from pykickstart.errors import KickstartParseError, KickstartParseWarning
from pykickstart.options import KSOptionParser

import warnings
from pykickstart.i18n import _

class F8_DeviceData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.moduleName = kwargs.get("moduleName", "")
        self.moduleOpts = kwargs.get("moduleOpts", "")

    def __eq__(self, y):
        if not y:
            return False

        return self.moduleName == y.moduleName

    def __ne__(self, y):
        return not self == y

    def __str__(self):
        retval = BaseData.__str__(self)

        if self.moduleName:
            retval += "device %s" % self.moduleName

            if self.moduleOpts:
                retval += " --opts=\"%s\"" % self.moduleOpts

        return retval + "\n"

class FC3_Device(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.type = kwargs.get("type", "")
        self.moduleName = kwargs.get("moduleName", "")
        self.moduleOpts = kwargs.get("moduleOpts", "")

    def __eq__(self, y):
        if not y:
            return False

        return self.moduleName == y.moduleName

    def __ne__(self, y):
        return not self == y

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.moduleName:
            retval += "device %s %s" % (self.type, self.moduleName)

            if self.moduleOpts:
                retval += " --opts=\"%s\"" % self.moduleOpts

        return retval + "\n"

    def _getParser(self):
        op = KSOptionParser(prog="device", description="""
            On most PCI systems, the installation program will autoprobe for
            Ethernet and SCSI cards properly. On older systems and some PCI
            systems, however, kickstart needs a hint to find the proper
            devices. The device command, which tells the installation program
            to install extra modules, is in this format:

            ``device <moduleName> --opts=<options>``

            ``<moduleName>``

            Replace with the name of the kernel module which should be
            installed.""", version=FC3)
        op.add_argument("--opts", dest="moduleOpts", default="", version=FC3,
                        help="""
                        Options to pass to the kernel module. For example:

                        ``--opts="aic152x=0x340 io=11"``
                        """)
        return op

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)

        if len(extra) != 2:
            raise KickstartParseError(_("device command requires two arguments: module type and name"), lineno=self.lineno)
        elif any(arg for arg in extra if arg.startswith("-")):
            mapping = {"command": "device", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)

        self.moduleOpts = ns.moduleOpts
        self.type = extra[0]
        self.moduleName = extra[1]
        return self

class F8_Device(FC3_Device):
    removedKeywords = FC3_Device.removedKeywords
    removedAttrs = FC3_Device.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        FC3_Device.__init__(self, writePriority, *args, **kwargs)
        self.deviceList = kwargs.get("deviceList", [])

    def __str__(self):
        retval = ""
        for device in self.deviceList:
            retval += device.__str__()

        return retval

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)

        if len(extra) != 1:
            raise KickstartParseError(_("%(command)s command requires a single argument: %(argument)s") % {"command": "device", "argument": "module name"}, lineno=self.lineno)
        elif any(arg for arg in extra if arg.startswith("-")):
            mapping = {"command": "device", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)

        dd = self.dataClass()   # pylint: disable=not-callable
        self.set_to_obj(ns, dd)
        dd.lineno = self.lineno
        dd.moduleName = extra[0]

        # Check for duplicates in the data list.
        if dd in self.dataList():
            warnings.warn(_("A module with the name %s has already been defined.") % dd.moduleName, KickstartParseWarning)

        return dd

    def dataList(self):
        return self.deviceList

    @property
    def dataClass(self):
        return self.handler.DeviceData

class F24_Device(DeprecatedCommand, F8_Device):
    def __init__(self):  # pylint: disable=super-init-not-called
        DeprecatedCommand.__init__(self)

    def _getParser(self):
        op = F8_Device._getParser(self)
        op.description += "\n\n.. deprecated:: %s" % versionToLongString(F24)
        return op

class F34_Device(RemovedCommand, F24_Device):
    def _getParser(self):
        op = F24_Device._getParser(self)
        op.description += "\n\n.. versionremoved:: %s" % versionToLongString(F34)
        return op



import unittest
from tests.baseclass import CommandTest


class F28_TestCase(CommandTest):
    command = "authselect"

    def runTest(self):
        # pass
        self.assert_parse("authselect")

        self.assert_parse("authselect select winbind",
                          "authselect select winbind\n")

        self.assert_parse("authselect select sssd with-mkhomedir",
                          "authselect select sssd with-mkhomedir\n")


if __name__ == "__main__":
    unittest.main()



import unittest
import shlex
import re

from pykickstart.parser import KickstartParser
from pykickstart.sections import NullSection
from pykickstart.version import makeVersion

VALID_KICKSTART_OPTION_PATTERN = r"--[a-z0-9\-]+"


class ArgumentNamesStatic_TestCase(unittest.TestCase):

    def setUp(self):
        self._bad_options = []
        self._reg_name_checker = re.compile(VALID_KICKSTART_OPTION_PATTERN)
        self._handler = makeVersion()

    def runTest(self):
        self._check_commands()
        self._check_sections()
        self._report()

    def _check_sections(self):

        tmp_parser = KickstartParser(self._handler)
        # KickstartParser registers internally all sections, so it is possible to get the names and
        # instances from there. However, it handles some known sections implemented outside
        # pykickstart by using NullSection instead of the actual class.

        for section, instance in tmp_parser._sections.items():
            if not isinstance(instance, NullSection):
                arg_parser = instance._getParser()
                self._check_parser_actions(arg_parser, section)

    def _check_commands(self):

        for command, cmdClass in self._handler.commandMap.items():

            if command == "method":
                continue

            args = shlex.split(command, comments=True)
            cmd = args[0]

            ks_parser = self._handler.commands[cmd]
            ks_parser.currentLine = command
            ks_parser.currentCmd = args[0]
            ks_parser.seen = True
            arg_parser = ks_parser._getParser()

            self._check_parser_actions(arg_parser, command, cmd_class=cmdClass)

    def _check_parser_actions(self, arg_parser, cmd_name, cmd_class=None):

        for action in arg_parser._get_optional_actions():
            if action.deprecated:
                continue

            found_any_good = False
            for option in action.option_strings:
                if cmd_class and option.lstrip("-") in cmd_class.removedAttrs:
                    # caution removedAttrs does not include leading dashes
                    continue
                is_good = self._reg_name_checker.fullmatch(option) is not None
                found_any_good = found_any_good or is_good
                if not is_good:
                    print("Found option with uppercase letters: %s for command %s." % (option, cmd_name))

            if not found_any_good:
                self._bad_options.append(tuple((cmd_name, action.option_strings)))

    def _report(self):
        if self._bad_options:
            print("The following kickstart option sets do not include a lowercase only variant:")
            for option_set in self._bad_options:
                print("%s: %s" % (option_set[0], ", ".join(option_set[1])))

            self.fail("Some options use uppercase letters and do not have a lowercase-only alias.")



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2007-2014 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
__all__ = ["commandMap", "dataMap"]

import os
import sys
import importlib

commandMap = {}
dataMap = {}

if not commandMap:
    _path = os.path.dirname(__file__)
    if not _path in sys.path:
        sys.path.append(_path)

    for name in os.listdir(os.path.dirname(__file__)):
        if not (name.startswith("fc") or name.startswith("f") or name.startswith("rhel")):
            continue

        if not name.endswith(".py"):
            continue

        obj = importlib.import_module(name.replace(".py", ""))
        if not obj.__all__ or not obj.__all__[0].endswith("Handler"):
            continue

        # Now we've got a WhateverHandler module in obj.  This module should
        # export one class named WhateverHandler, which we can get at indirectly
        # through __all__, like so:
        handler = obj.__dict__[obj.__all__[0]]
        commandMap[handler.version] = handler.commandMap
        dataMap[handler.version] = handler.dataMap



#
# Copyright 2020 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.errors import KickstartParseError
from pykickstart.version import F33
from pykickstart.base import BaseData, KickstartCommand
from pykickstart.options import KSOptionParser

from pykickstart.i18n import _

class F33_TimesourceData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.ntp_server = kwargs.get("ntp_server", "")
        self.ntp_pool = kwargs.get("ntp_pool", "")
        self.ntp_disable = kwargs.get("ntp_disable", False)
        self.nts = kwargs.get("nts", False)

    def __eq__(self, y):
        if not y:
            return False

        if self.ntp_server:
            return self.ntp_server == y.ntp_server and self.nts == y.nts
        elif self.ntp_pool:
            return self.ntp_pool == y.ntp_pool and self.nts == y.nts
        else:
            # given that one of ntp_server/ntp_pool/ntp_disable is required
            # this will be --ntp_disable & such timesource invocations will
            # always be the same by definition
            return True

    def __ne__(self, y):
        return not self == y

    def __str__(self):
        retval = BaseData.__str__(self)

        args = self._getArgsAsStr()
        if args:
            retval += "timesource%s\n" % args

        return retval

    def _getArgsAsStr(self):
        retval = ""

        if self.ntp_server:
            retval += " --ntp-server=%s" % self.ntp_server
        if self.ntp_pool:
            retval += " --ntp-pool=%s" % self.ntp_pool
        if self.nts:
            retval += " --nts"
        if self.ntp_disable:
            retval += " --ntp-disable"

        return retval


class F33_Timesource(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()
        self.timesource_list = kwargs.get("timesource_list", [])
        self.exclusive_required_options = [("ntp_server", "--ntp-server"),
                                           ("ntp_pool", "--ntp-pool"),
                                           ("ntp_disable", "--ntp-disable")]

    def __str__(self):
        retval = ""
        for timesource_command in self.timesource_list:
            retval += timesource_command.__str__()

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="timesource", description="""
                            Configures a timesource.""",
                            version=F33)
        op.add_argument("--ntp-server", version=F33, help="""
                        A single NTP server.

                        ``--ntp-server``, ``--ntp-pool`` and ``--ntp-disable`` are mutually exclusive.
                        """)
        op.add_argument("--ntp-pool", version=F33, help="""
                        A single NTP pool.

                        ``--ntp-pool``, ``--ntp-server`` and ``--ntp-disable`` are mutually exclusive.
                        """)
        op.add_argument("--ntp-disable", action="store_true",
                        default=False, version=F33, help="""
                        If specified, disable any NTP based time sync,
                        both on target system as well as in installation
                        environment.
                        """)
        op.add_argument("--nts", dest="nts", action="store_true",
                        default=False, version=F33, help="""
                        If specified, consider the provided hostname to be
                        a NTS compatible time source. Without the ``--nts``
                        option it will be considered to be a plain NTP time
                        source without NTS support.
                        """)
        return op

    def parse(self, args):
        td = self.dataClass()   # pylint: disable=not-callable
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_obj(ns, td)
        td.lineno = self.lineno

        # Check that just one of exclusive required options is specified
        used_options = [opt for attr, opt in self.exclusive_required_options
                        if getattr(ns, attr, None)]
        if len(used_options) == 0:
            mapping = {"options_list": ", ".join((opt for attr, opt in self.exclusive_required_options))}
            raise KickstartParseError(_("One of -%(options_list)s options must be specified for timesource command.") % mapping, lineno=self.lineno)
        if len(used_options) > 1:
            mapping = {"options_list": ", ".join((opt for opt in used_options))}
            raise KickstartParseError(_("Only one of %(options_list)s options may be specified for timesource command.") % mapping, lineno=self.lineno)

        return td

    def dataList(self):
        return self.timesource_list

    @property
    def dataClass(self):
        return self.handler.TimesourceData



#
# Alexander Todorov <atodorov@redhat.com>
#
# Copyright 2016 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.base import DeprecatedCommand, RemovedCommand
from pykickstart.version import F20, F29, F34, versionToLongString
from pykickstart.commands.upgrade import F11_Upgrade


class F20_Install(F11_Upgrade):
    """
        The upgrade command has been deprecated in Fedora 20. This
        class should be used for any further updates to the install
        command. This separation is required because there's no
        clean way of deprecating only half of the behavior of some
        command class and still have the handlers map to the latest
        possible version of this class!
    """
    def _getParser(self):
        op = super(F20_Install, self)._getParser()
        op.prog = "install"
        op.description = """
            Install a fresh system. You must specify the type of
            installation from one of cdrom, harddrive, nfs, or url
            (for ftp or http installations).
            The install command and the installation method command
            must be on separate lines.

            Important: before Fedora 20 this command was known as
            install or upgrade but the upgrade part was deprecated!
        """
        op.version = F20

        return op

    def parse(self, args):
        super(F20_Install, self).parse(args)
        # since F20 we always return False for upgrades
        self.upgrade = False

        return self


class F29_Install(DeprecatedCommand, F20_Install):
    def __init__(self):  # pylint: disable=super-init-not-called
        DeprecatedCommand.__init__(self)

    def _getParser(self):
        op = F20_Install._getParser(self)
        op.description += "\n\n.. deprecated:: %s" % versionToLongString(F29)
        return op


class F34_Install(RemovedCommand, F29_Install):
    def _getParser(self):
        op = F29_Install._getParser(self)
        op.description += "\n\n.. versionremoved:: %s" % versionToLongString(F34)
        return op



import os
import sys
try:
    from io import StringIO
except ImportError:
    from cStringIO import StringIO
from tools import ksflatten
from pykickstart import __version__
import unittest.mock as mock
from unittest import TestCase
from unittest import skipUnless
from contextlib import contextmanager
from tests.tools.utils import mktempfile


@contextmanager
def capture(command, *args, **kwargs):
    stdout, sys.stdout = sys.stdout, StringIO()
    stderr, sys.stderr = sys.stderr, StringIO()
    try:
        retval = command(*args, **kwargs)
        sys.stdout.seek(0)
        sys.stderr.seek(0)
        yield (retval, sys.stdout.read(), sys.stderr.read())
    finally:
        sys.stdout = stdout
        sys.stderr = stderr


class NoConfigSpecified_TestCase(TestCase):
    @mock.patch('sys.exit', create=True)
    def runTest(self, _exit):
        with capture(ksflatten.main, []) as output:
            retval, msg = output[0] 
            self.assertEqual(retval, 1)
            self.assertEqual(msg, 'Need to specify a config to flatten')

class WrongKSPath_TestCase(TestCase):
    def runTest(self):
        retval, msg = ksflatten.main(['--version', 'F26', '--config', '/non/existing'])
        self.assertEqual(retval, 1)
        self.assertTrue(msg.find('Failed to parse kickstart file') > -1)

class ReadKickstartIOError_TestCase(TestCase):
    @mock.patch('pykickstart.parser.KickstartParser.readKickstart')
    def runTest(self, _readKickstart):
        _readKickstart.side_effect = IOError
        retval, msg = ksflatten.main(['--version', 'F26', '--config', '/dev/null'])
        self.assertEqual(retval, 1)
        self.assertTrue(msg.find('Failed to read kickstart file') > -1)

class ValidKSFile_ToStdOut_TestCase(TestCase):
    def setUp(self):
        self._include_path = mktempfile("text", prefix="ks-include")
        ks_content = "autopart\n%%include %s" % self._include_path
        self._ks_path = mktempfile(ks_content)

    def tearDown(self):
        os.unlink(self._ks_path)
        os.unlink(self._include_path)

    def runTest(self):
        with capture(ksflatten.main, ['--version', 'F26', '--config', self._ks_path]) as output:
            self.assertEqual(output[1], "# Generated by pykickstart v%s\n#version=F26\n# Use text mode install\ntext\n\n# System bootloader configuration\nbootloader --location=none\nautopart\n" % __version__)

class ValidKSFile_TestCase(TestCase):
    def setUp(self):
        self._include_path = mktempfile("text", prefix="ks-include")
        ks_content = "autopart\n%%include %s" % self._include_path
        self._ks_path = mktempfile(ks_content)
        self._output_path = mktempfile()

    def tearDown(self):
        os.unlink(self._ks_path)
        os.unlink(self._include_path)
        os.unlink(self._output_path)

    def runTest(self):
        retval, msg = ksflatten.main(['--version', 'F26',
                                      '--config', self._ks_path,
                                      '--output', self._output_path])
        self.assertEqual(retval, 0)
        self.assertEqual(msg, "")
        f = open(self._output_path, 'r')
        output = f.read()
        f.close()
        self.assertEqual(output, "# Generated by pykickstart v%s\n#version=F26\n# Use text mode install\ntext\n\n# System bootloader configuration\nbootloader --location=none\nautopart\n" % __version__)

@skipUnless(os.getuid(), "test requires non-root access")
class FailsToOpenOutputFile_TestCase(TestCase):
    def setUp(self):
        self._include_path = mktempfile("text", prefix="ks-include")
        ks_content = "autopart\n%%include %s" % self._include_path
        self._ks_path = mktempfile(ks_content)

        self._output_path = mktempfile()
        # create IOError
        os.chmod(self._output_path, 0o000)

    def tearDown(self):
        os.unlink(self._ks_path)
        os.unlink(self._include_path)
        os.unlink(self._output_path)

    def runTest(self):
        retval, msg = ksflatten.main(['--version', 'F26',
                                      '--config', self._ks_path,
                                      '--output', self._output_path])
        self.assertEqual(retval, 1)
        self.assertTrue(msg.find('Failed to open output file') > -1)



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from textwrap import dedent

from pykickstart.version import F7, F34, versionToLongString
from pykickstart.base import KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser

from pykickstart.i18n import _

class F7_Updates(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()
        self.url = kwargs.get("url", "")

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.url == "floppy":
            retval += "updates\n"
        elif self.url:
            retval += "updates %s\n" % self.url

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="updates", description="""
                            Specify the location of an updates.img for use in
                            installation. See anaconda-release-notes.txt for a
                            description of how to make an updates.img.""",
                            version=F7)
        op.add_argument("updates", metavar="[URL]", nargs="*", version=F7,
                        help="""
                        If present, the URL for an updates image.

                        If not present, anaconda will attempt to load from a
                        floppy disk.""")
        return op

    def parse(self, args):
        (_ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)

        if len(_ns.updates) > 1:
            raise KickstartParseError(_("Kickstart command %s only takes one argument") % "updates", lineno=self.lineno)
        elif extra:
            mapping = {"command": "updates", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)
        elif not _ns.updates:
            self.url = "floppy"
        else:
            self.url = _ns.updates[0]

        return self


class F34_Updates(F7_Updates):
    removedKeywords = F7_Updates.removedKeywords
    removedAttrs = F7_Updates.removedAttrs

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.url:
            retval += "updates %s\n" % self.url

        return retval

    def _getParser(self):
        op = KSOptionParser(
            prog="updates", version=F7, description="""
            Specify the location of an updates.img for use in installation.
            """
        )
        url_action = op.add_argument(
            "url", metavar="[URL]", version=F7, help="""
            If present, the URL for an updates image.

            If not present, anaconda will attempt to load from a floppy disk.
            """
        )
        url_action.help += dedent(
            """

            .. versionchanged:: %s

            The URL for an updates image is required. Anaconda no longer supports
            updates on a floppy disk.
            """ % versionToLongString(F34)
        )
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        return self



import unittest
from tests.baseclass import CommandTest
from pykickstart.constants import SELINUX_DISABLED, SELINUX_ENFORCING, SELINUX_PERMISSIVE

class FC3_TestCase(CommandTest):
    command = "selinux"

    def runTest(self):
        # pass
        self.assert_parse("selinux")
        self.assert_parse("selinux --permissive", "selinux --permissive\n")
        self.assert_parse("selinux --enforcing", "selinux --enforcing\n")
        self.assert_parse("selinux --disabled", "selinux --disabled\n")

        # fail
        self.assert_parse_error("selinux --cheese")
        self.assert_parse_error("selinux --crackers=CRUNCHY")

        # extra test coverage
        cmd = self.handler().commands[self.command]
        for mode in [-1, 99999]:
            cmd.selinux = mode
            self.assertEqual(cmd.__str__(), "# SELinux configuration\n")

        cmd.selinux = SELINUX_DISABLED
        self.assertEqual(cmd.__str__(), "# SELinux configuration\nselinux --disabled\n")

        cmd.selinux = SELINUX_ENFORCING
        self.assertEqual(cmd.__str__(), "# SELinux configuration\nselinux --enforcing\n")

        cmd.selinux = SELINUX_PERMISSIVE
        self.assertEqual(cmd.__str__(), "# SELinux configuration\nselinux --permissive\n")

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
"""
Base classes for internal pykickstart use.

The module exports the following important classes:

    KickstartObject - The base class for all classes in pykickstart
"""

class KickstartObject(object):
    """The base class for all other classes in pykickstart."""
    def __init__(self, *args, **kwargs):
        """Create a new KickstartObject instance.  All other classes in
           pykickstart should be derived from this one.  Instance attributes:
        """

    def __str__(self):
        return ""



#
# Peter Jones <pjones@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import unittest

from pykickstart.errors import KickstartParseWarning
from tests.baseclass import CommandTest, CommandSequenceTest
from pykickstart.commands.sshpw import F13_SshPwData, F13_SshPw, F24_SshPwData
from pykickstart.version import F13

class SshPw_TestCase(unittest.TestCase):
    def runTest(self):
        data1 = F13_SshPwData()
        data2 = F13_SshPwData()
        self.assertEqual(data1, data2)
        self.assertFalse(data1 != data2)
        self.assertNotEqual(data1, None)

        # assert default values
        self.assertFalse(data1.isCrypted)
        self.assertFalse(data1.lock)

        # test for objects difference
        for atr in ['username']:
            setattr(data1, atr, None)
            setattr(data2, atr, 'test')
            # objects that differ in only one attribute
            # are not equal
            self.assertNotEqual(data1, data2)
            self.assertNotEqual(data2, data1)
            setattr(data1, atr, None)
            setattr(data2, atr, None)

        # check options defaults
        for action in F13_SshPw()._getParser()._actions:
            for a in ['--username']:
                if a in action.option_strings:
                    self.assertTrue(action.required)

            for a in ['--lock']:
                if a in action.option_strings:
                    self.assertFalse(action.default)

        # assert default values
        self.assertFalse(F24_SshPwData().sshkey)

class F13_TestCase(CommandTest):
    command = "sshpw"

    def runTest(self):
        # pass
        self.assert_parse("sshpw --username=someguy --iscrypted secrethandshake", "sshpw --username=someguy --iscrypted secrethandshake\n")

        self.assertFalse(self.assert_parse("sshpw --username=A --iscrypted secrethandshake") is None)
        self.assertTrue(self.assert_parse("sshpw --username=A --iscrypted secrethandshake") !=
                        self.assert_parse("sshpw --username=B --iscrypted secrethandshake"))
        self.assertFalse(self.assert_parse("sshpw --username=A --iscrypted secrethandshake") ==
                         self.assert_parse("sshpw --username=B --iscrypted secrethandshake"))

        # fail
        self.assert_parse_error("sshpw")
        self.assert_parse_error("sshpw --username=someguy --bogus-option")
        self.assert_parse_error("sshpw --username=someguy pass-phrase --bogus-option")
        self.assert_parse_error("sshpw --username=someguy")
        self.assert_parse_error("sshpw --username=someguy --iscrypted=OMGSEKRITZ")
        self.assert_parse_error("sshpw --username=someguy --iscrypted")

        # pass
        self.assert_parse("sshpw --username=someguy --lock secrethandshake", "sshpw --username=someguy --lock --plaintext secrethandshake\n")
        self.assert_parse("sshpw --username=someguy --plaintext secrethandshake", "sshpw --username=someguy --plaintext secrethandshake\n")
        self.assert_parse("sshpw --username=someguy --plaintext --iscrypted secrethandshake", "sshpw --username=someguy --iscrypted secrethandshake\n")
        self.assert_parse("sshpw --username=someguy --iscrypted --plaintext secrethandshake\n", "sshpw --username=someguy --plaintext secrethandshake\n")
        self.assert_parse("sshpw --username=someguy --lock --plaintext secrethandshake", "sshpw --username=someguy --lock --plaintext secrethandshake\n")
        self.assert_parse("sshpw --username=someguy --iscrypted --lock secrethandshake", "sshpw --username=someguy --lock --iscrypted secrethandshake\n")
        self.assert_parse("sshpw --username=someguy --lock --iscrypted --plaintext secrethandshake", "sshpw --username=someguy --lock --plaintext secrethandshake\n")
        self.assert_parse("sshpw --username=someguy --lock --plaintext --iscrypted secrethandshake", "sshpw --username=someguy --lock --iscrypted secrethandshake\n")
        self.assert_parse("sshpw --username=someguy --plaintext --iscrypted --lock secrethandshake", "sshpw --username=someguy --lock --iscrypted secrethandshake\n")
        self.assert_parse("sshpw --username=someguy --iscrypted --plaintext --lock secrethandshake", "sshpw --username=someguy --lock --plaintext secrethandshake\n")

        # fail
        self.assert_parse_error("sshpw --username=someguy --plaintext=ISEEENGLAND secrethandshake")
        self.assert_parse_error("sshpw --username=someguy --lock=NOKEYSFORYOU secrethandshake")
        self.assert_parse_error("sshpw --username=someguy --plaintext")
        self.assert_parse_error("sshpw --username=someguy --lock")

        # extra test coverage
        sshpw = self.handler().commands[self.command]
        sshpw.sshUserList.append("someguy")
        self.assertEqual(sshpw.__str__(), "someguy")

class F13_Duplicate_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = F13

    def runTest(self):
        self.assert_parse("""
sshpw --username=someguy --iscrypted passwordA
sshpw --username=otherguy --iscrypted passwordA""")

        self.assert_parse_error("""
sshpw --username=someguy --iscrypted passwordA
sshpw --username=someguy --iscrypted passwordB""", KickstartParseWarning)

class F24_TestCase(F13_TestCase):
    def runTest(self):
        self.assert_parse("sshpw --username=someguy --sshkey a ssh key with spaces", "sshpw --username=someguy --sshkey a ssh key with spaces\n")
        self.assert_parse("sshpw --username=someguy a password with spaces", "sshpw --username=someguy --plaintext a password with spaces\n")
        self.assert_parse("sshpw --username=someguy --lock secretpassword", "sshpw --username=someguy --lock --plaintext secretpassword\n")
        self.assert_parse("sshpw --username=someguy --iscrypted secretpassword", "sshpw --username=someguy --iscrypted secretpassword\n")

if __name__ == "__main__":
    unittest.main()



import os
import sys
import unittest
import importlib
from unittest import mock
import pykickstart
from pykickstart.options import KSOptionParser
from pykickstart.base import KickstartCommand, BaseData

class ClassDefinitionTestCase(unittest.TestCase):
    """
        Search for command and data classes defined like so:

        RHEL7_AutoPart = F21_AutoPart.

        This kind of definitions makes it possible for other
        tests to omit possible errors and we don't like to have
        them. Either we use the existing class name if we haven't
        redefined anything or provide a boilerplate definition:

        class RHEL7_Autopart(F21_Autopart):
            pass
    """
    def runTest(self):
        errors = 0
        commands_dir = os.path.join(os.path.dirname(pykickstart.__file__), "commands")
        commands_dir = os.path.abspath(commands_dir)

        self.assertTrue(os.path.exists(commands_dir))
        if commands_dir not in sys.path:
            sys.path.append(commands_dir)

        for _dirpath, _dirnames, paths in os.walk(commands_dir):
            for path in paths:
                if path == '__init__.py' or not path.endswith('.py'):
                    continue

                # load the module defining all possible command implementations
                command_module = importlib.import_module(path.replace(".py", ""))
                module_commands = []  # a list of already checked commands

                for impl_name, impl_class in command_module.__dict__.items():
                    # skip everything which isn't a class
                    if type(impl_class) is not type:
                        continue

                    # skip everything which doesn't inherit
                    # from KickstartCommand or BaseData
                    if not (issubclass(impl_class, KickstartCommand) or issubclass(impl_class, BaseData)):
                        continue

                    # skip base classes as well
                    if impl_class.__name__ in ['KickstartCommand', 'DeprecatedCommand', 'RemovedCommand']:
                        continue

                    if impl_class not in module_commands and \
                        impl_class.__name__ == impl_name:
                        module_commands.append(impl_class)
                    else:
                        errors += 1
                        message = "ERROR: In `commands/%s` %s = %s" % (path, impl_name, impl_class.__name__)
                        print(message)

        # assert for errors presence
        self.assertEqual(0, errors)

class TestKSOptionParser(KSOptionParser):
    """
        Wrapper class that will raise exception if some of the
        help attributes are empty.
    """
    def __init__(self, *args, **kwargs):
        self._test_errors = []
        for arg_name in ['prog', 'version', 'description']:
            if not kwargs.get(arg_name) and not kwargs.get("deprecated"):
                self._test_errors.append("%s: %s can't be blank" % (args[0], arg_name))

        super(TestKSOptionParser, self).__init__(*args, **kwargs)

    def add_argument(self, *args, **kwargs):
        for arg_name in ['help', 'version']:
            if not kwargs.get(arg_name) and not kwargs.get("deprecated"):
                self._test_errors.append("%s: %s can't be blank" % (args[0], arg_name))
        return super(TestKSOptionParser, self).add_argument(*args, **kwargs)


class HelpAndDescription_TestCase(unittest.TestCase):
    """
        Check that all commands and their options have some description text.
    """

    def runTest(self):
        errors = 0
        commands_dir = os.path.join(os.path.dirname(pykickstart.__file__), "commands")
        commands_dir = os.path.abspath(commands_dir)

        self.assertTrue(os.path.exists(commands_dir))
        if commands_dir not in sys.path:
            sys.path.append(commands_dir)

        for _dirpath, _dirnames, paths in os.walk(commands_dir):
            for path in paths:
                if path == '__init__.py' or not path.endswith('.py'):
                    continue

                # load the module defining all possible command implementations
                command_module = importlib.import_module(path.replace(".py", ""))

                for _, impl_class in command_module.__dict__.items():
                    # skip everything which isn't a class
                    if type(impl_class) is not type:
                        continue

                    # skip everything which doesn't inherit from KickstartCommand
                    if not issubclass(impl_class, KickstartCommand):
                        continue

                    # skip base classes as well
                    if impl_class.__name__ in ['KickstartCommand', 'DeprecatedCommand', 'RemovedCommand']:
                        continue

                    # In order for patch to locate the function to be patched, it must be
                    # specified using its fully qualified name, which may not be what you expect.
                    # For example, if a class is imported in the module my_module.py as follows:
                    # from module import ClassA
                    # It must be patched as patch(my_module.ClassA), rather than patch(module.ClassA),
                    # due to the semantics of the from ... import ... statement, which imports
                    # classes and functions into the current namespace.
                    command_module_name = command_module.__name__
                    # the install.py command inherits from upgrade.py and doesn't import
                    # KSOptionParser on its own
                    if command_module_name == 'install':
                        command_module_name = 'upgrade'
                    with mock.patch('%s.KSOptionParser' % command_module_name, new=TestKSOptionParser):
                        # just construct the option parser
                        # the wrapper class will raise an exception in case
                        # there are empty help strings
                        klass = impl_class()
                        op = klass._getParser()

                        if hasattr(op, "_test_errors") and len(op._test_errors) > 0:
                            errors += len(op._test_errors)
                            print("ERROR: In `%s`" % impl_class)
                            for err in op._test_errors:
                                print(err)

        # assert for errors presence
        self.assertEqual(0, errors)

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import versionToLongString, RHEL3, FC3, F7
from pykickstart.base import DeprecatedCommand, KickstartCommand, RemovedCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser

from pykickstart.i18n import _

class RHEL3_Mouse(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.device = kwargs.get("device", "")
        self.emulthree = kwargs.get("emulthree", False)
        self.mouse = kwargs.get("mouse", "")

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        opts = ""
        if self.device:
            opts += "--device=%s " % self.device
        if self.emulthree:
            opts += "--emulthree "

        if self.mouse:
            retval += "# System mouse\nmouse %s%s\n" % (opts, self.mouse)
        return retval

    def _getParser(self):
        op = KSOptionParser(prog="mouse", description="""
                            Configure the system mouse""", version=RHEL3)
        op.add_argument("--device", default="", version=RHEL3,
                        help="Which device node to use for mouse")
        op.add_argument("--emulthree", default=False, action="store_true",
                        version=RHEL3, help="If set emulate 3 mouse buttons")
        return op

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)

        if len(extra) != 1:
            raise KickstartParseError(_("Kickstart command %s requires one argument") % "mouse", lineno=self.lineno)
        elif any(arg for arg in extra if arg.startswith("-")):
            mapping = {"command": "mouse", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)

        self.set_to_self(ns)
        self.mouse = extra[0]
        return self

class FC3_Mouse(DeprecatedCommand, RHEL3_Mouse):
    def __init__(self):  # pylint: disable=super-init-not-called
        DeprecatedCommand.__init__(self)

    def _getParser(self):
        op = RHEL3_Mouse._getParser(self)
        op.description += "\n\n.. deprecated:: %s" % versionToLongString(FC3)
        return op

class F7_Mouse(RemovedCommand, FC3_Mouse):
    def _getParser(self):
        op = FC3_Mouse._getParser(self)
        op.description += "\n\n.. versionremoved:: %s" % versionToLongString(F7)
        return op



#
# James Laska <jlaska@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import unittest
from tests.baseclass import CommandTest
from pykickstart.commands.autostep import FC3_AutoStep
from pykickstart.base import DeprecatedCommand

class FC3_TestCase(CommandTest):
    command = "autostep"

    def runTest(self):
        # pass
        self.assert_parse("autostep", "autostep\n")
        self.assert_parse("autostep --autoscreenshot", "autostep --autoscreenshot\n")

        # fail
        self.assert_parse_error("autostep --autoscreenshot=FOO")

class F34_TestCase(FC3_TestCase):
    def runTest(self):
        # make sure we've been deprecated
        parser = self.getParser("autostep")
        self.assertTrue(issubclass(parser.__class__, DeprecatedCommand))

        # make sure we are still able to parse it
        self.assert_parse("autostep")

class AutoStep_TestCase(unittest.TestCase):
    """
        Additional autostep tests
    """
    def runTest(self):
        # by default autostep is False if not specified
        cmd = FC3_AutoStep()
        self.assertEqual(cmd.autostep, False)
        self.assertEqual(cmd.autoscreenshot, False)

        # after parsing autostep becomes True
        cmd.parse(['--autoscreenshot'])
        self.assertEqual(cmd.autostep, True)
        self.assertEqual(cmd.autoscreenshot, True)

if __name__ == "__main__":
    unittest.main()



#
# Copyright 2020 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest

from pykickstart.version import F33
from tests.baseclass import CommandTest, CommandSequenceTest
from pykickstart.commands.timesource import F33_TimesourceData

class F33_TestCase(CommandTest):
    command = "timesource"

    def runTest(self):
        data1 = F33_TimesourceData()
        data1.ntp_server="ntp.example.com"
        data2 = F33_TimesourceData()
        data2.ntp_server="ntp.example.com"
        self.assertEqual(data1, data2)
        self.assertFalse(data1 != data2) # test __ne__ method
        self.assertNotEqual(data1, None)

        for attr in ['ntp_server', 'ntp_pool']:
            data1 = F33_TimesourceData()
            data2 = F33_TimesourceData()
            setattr(data1, attr, 'foo')
            setattr(data2, attr, 'test')
            self.assertNotEqual(data1, data2)
            self.assertNotEqual(data2, data1)
            setattr(data1, attr, '')
            setattr(data2, attr, '')

        # pass
        self.assert_parse("timesource --ntp-server=ntp.example.com")
        self.assert_parse("timesource --ntp-pool=pool.example.com")
        self.assert_parse("timesource --ntp-server=ntp.example.com --nts")
        self.assert_parse("timesource --ntp-pool=pool.example.com --nts")
        self.assert_parse("timesource --ntp-disable")

        # fail
        # server and pool can't be used at the same time
        self.assert_parse_error("timesource --ntp-server=ntp.example.com --ntp-pool=pool.example.com")
        self.assert_parse_error("timesource --ntp-server=ntp.example.com --ntp-pool=pool.example.com --nts")
        # disable can't be used with neither server or pool
        self.assert_parse_error("timesource --ntp-disable --ntp-pool=pool.example.com")
        self.assert_parse_error("timesource --ntp-server=ntp.example.com --ntp-disable")
        # --ntp-server requires an argument
        self.assert_parse_error("timesource --ntp-server")
        self.assert_parse_error("timesource --ntp-server --nts")
        # --ntp-pool requires int argument
        self.assert_parse_error("timesource --ntp-pool")
        self.assert_parse_error("timesource --ntp-pool --nts")
        # unknown option
        self.assert_parse_error("timesource --ntp-server=ntp.example.com --unknown=value")
        # required option arguments
        self.assert_parse_error("timesource")
        self.assert_parse_error("timesource --nts")

        # test output kickstart generation
        self.assert_parse('timesource --ntp-server=ntp.example.com',
                          'timesource --ntp-server=ntp.example.com\n')
        self.assert_parse('timesource --ntp-pool=pool.example.com',
                          'timesource --ntp-pool=pool.example.com\n')
        self.assert_parse('timesource --ntp-server=ntp.example.com --nts',
                          'timesource --ntp-server=ntp.example.com --nts\n')
        self.assert_parse('timesource --ntp-pool=pool.example.com --nts',
                          'timesource --ntp-pool=pool.example.com --nts\n')
        self.assert_parse('timesource --ntp-disable',
                          'timesource --ntp-disable\n')

        # extra test coverage
        td = self.handler().TimesourceData()
        td.ntp_server = ""
        td.ntp_pool = ""
        self.assertEqual(td.__str__(), "")
        self.assertEqual(td._getArgsAsStr(), "")

        cmd = self.handler().commands[self.command]
        cmd.timesource_list = [td]
        self.assertEqual(cmd.__str__(), "")


class F33_MultiCommand_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = F33

    def runTest(self):
        self.assert_parse("""
timesource --ntp-server=ntp.example.com
timesource --ntp-pool=pool.example.com
timesource --ntp-disable
""")

        self.assert_parse_error("""
timesource --ntp-server=ntp.example.com
timesource --ntp-pool=pool.example.com
timesource --nts
""")

        self.assert_parse_error("""
timesource --ntp-server=ntp.example.com
timesource --ntp-pool=pool.example.com
timesource --ntp-server=ntp.example.com --ntp-pool=pool.example.com
""")


if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2007, 2009, 2013 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import F19
from pykickstart.base import KickstartCommand
from pykickstart.options import KSOptionParser

class F19_Liveimg(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.checksum = kwargs.get("checksum", "")
        self.noverifyssl = kwargs.get("noverifyssl", None)
        self.proxy = kwargs.get("proxy", None)
        self.url = kwargs.get("url", None)

        self.op = self._getParser()

    def __eq__(self, other):
        if not other:
            return False

        return self.url == other.url and self.proxy == other.proxy and \
               self.noverifyssl == other.noverifyssl and \
               self.checksum == other.checksum

    def __str__(self):
        retval = KickstartCommand.__str__(self)
        if not self.seen:
            return retval

        retval += "# Use live disk image installation\n"

        retval += "liveimg --url=\"%s\"" % self.url

        if self.proxy:
            retval += " --proxy=\"%s\"" % self.proxy

        if self.noverifyssl:
            retval += " --noverifyssl"

        if self.checksum:
            retval += " --checksum=\"%s\"" % self.checksum

        return retval + "\n"

    def _getParser(self):
        op = KSOptionParser(prog="liveimg", description="""
            Install a disk image instead of packages. The image can be the
            squashfs.img from a Live iso, or any filesystem mountable by the
            install media (eg. ext4). Anaconda expects the image to contain
            utilities it needs to complete the system install so the best way to
            create one is to use livemedia-creator to make the disk image. If
            the image contains /LiveOS/\\*.img (this is how squashfs.img is
            structured) the first \\*.img file inside LiveOS will be mounted and
            used to install the target system. The URL may also point to a
            tarfile of the root filesystem. The file must end in .tar, .tbz,
            .tgz, .txz, .tar.bz2, tar.gz, tar.xz""", version=F19)
        op.add_argument("--url", metavar="<url>", required=True, version=F19,
                        help="""
                        The URL to install from. http, https, ftp and file are
                        supported.""")
        op.add_argument("--proxy", metavar="<proxyurl>", version=F19, help="""
                        Specify an HTTP/HTTPS/FTP proxy to use while performing
                        the install. The various parts of the argument act like
                        you would expect. Syntax is::

                        ``--proxy=[protocol://][username[:password]@]host[:port]``
                        """)
        op.add_argument("--noverifyssl", action="store_true", version=F19,
                        default=False, help="""
                        For a tree on a HTTPS server do not check the server's
                        certificate with what well-known CA validate and do not
                        check the server's hostname matches the certificate's
                        domain name.""")
        op.add_argument("--checksum", metavar="<sha256>", version=F19,
                        help="Optional sha256 checksum of the image file")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        return self



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import RHEL5
from pykickstart.base import KickstartCommand
from pykickstart.constants import KS_INSTKEY_SKIP
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser

from pykickstart.i18n import _

class RHEL5_Key(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()
        self.key = kwargs.get("key", "")
        self.skip = kwargs.get("skip", False)

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.key == KS_INSTKEY_SKIP:
            retval += "key --skip\n"
        elif self.key:
            retval += "key %s\n" % self.key

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="key", description="set or skip the software installation key", version=RHEL5)
        op.add_argument("--skip", action="store_true", default=False,
                        version=RHEL5, help="Skip the software installation key")
        return op

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)

        if self.skip:
            self.key = KS_INSTKEY_SKIP
        elif len(extra) != 1:
            raise KickstartParseError(_("Kickstart command %s requires one argument") % "key", lineno=self.lineno)
        elif any(arg for arg in extra if arg.startswith("-")):
            mapping = {"command": "key", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)
        else:
            self.key = extra[0]

        return self



#
# Martin Gracik <mgracik@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from tests.baseclass import CommandTest
from pykickstart.commands.firewall import FC3_Firewall, F10_Firewall, F20_Firewall, F28_Firewall

class Firewall_TestCase(unittest.TestCase):
    def runTest(self):
        cmd = FC3_Firewall()
        self.assertEqual(cmd.__str__(), '')
        self.assertEqual(F10_Firewall().__str__(), '')
        self.assertEqual(F20_Firewall().__str__(), '')
        self.assertEqual(F28_Firewall().__str__(), '')

        op = cmd._getParser()
        for action in op._actions:
            if '--enable' in action.option_strings:
                self.assertTrue(action.default)

class FC3_TestCase(CommandTest):
    command = "firewall"

    def runTest(self):
        # pass
        # no colon in --port string
        self.assert_parse("firewall --enabled --port=47",
                          "firewall --enabled --port=47:tcp\n")

        # enable firewall
        if "--service" in self.optionList:
            self.assert_parse("firewall --enabled --trust=eth0 --ssh --port=imap:tcp",
                              "firewall --enabled --port=imap:tcp --trust=eth0 --service=ssh\n")
            self.assert_parse("firewall --enabled --ssh --ftp", "firewall --enabled --service=ssh,ftp\n")
        else:
            self.assert_parse("firewall --enabled --trust=eth0 --ssh --port=imap:tcp",
                              "firewall --enabled --port=22:tcp,imap:tcp --trust=eth0\n")
            self.assert_parse("firewall --enable --port=1234:udp,4321:tcp", "firewall --enabled --port=1234:udp,4321:tcp\n")

            # s-c-ks could have passed this sort of string in, so we need to test the conversion
            self.assert_parse("firewall --enabled --port=ssh,telnet,smtp,http,ftp",
                              "firewall --enabled --ssh --telnet --smtp --http --ftp\n")

        if "--telnet" in self.optionList:
            self.assert_parse("firewall --enable --trust=eth0,eth1 --ssh --telnet --http --smtp --ftp --port=1234:udp"
                              "firewall --enabled --port=22:tcp,23:tcp,80:tcp,443:tcp,25:tcp,21:tcp,1234:udp --trust=eth0,eth1\n")
        elif "--service" in self.optionList:
            self.assert_parse("firewall --enable --trust=eth0,eth1 --ssh --http --smtp --ftp --port=1234:udp"
                              "firewall --enabled --port=1234:udp --trust=eth0,eth1 --service=ssh,http,smtp,ftp\n")

        # disable firewall
        self.assert_parse("firewall --disabled", "firewall --disabled\n")
        self.assert_parse("firewall --disable", "firewall --disabled\n")

        # enable by default
        self.assert_parse("firewall --trust=eth0", "firewall --enabled --trust=eth0\n")
        self.assert_parse("firewall", "firewall --enabled\n")

        # deprecated
        if "--high" in self.optionList:
            self.assert_deprecated("firewall", "--high")
        if "--medium" in self.optionList:
            self.assert_deprecated("firewall", "--medium")

        # fail
        # unknown option
        self.assert_parse_error("firewall --bad-flag")
        # unexpected argument
        self.assert_parse_error("firewall arg")

class F9_TestCase(FC3_TestCase):
    def runTest(self):
        # run FC3 test case
        FC3_TestCase.runTest(self)

        # removed
        self.assert_removed("firewall", "--high")
        self.assert_removed("firewall", "--medium")

class F10_TestCase(F9_TestCase):
    def runTest(self):
        F9_TestCase.runTest(self)

        # deprecated
        self.assert_deprecated("firewall", "--telnet")

class F14_TestCase(F10_TestCase):
    def runTest(self):
        F10_TestCase.runTest(self)

        # removed
        self.assert_removed("firewall", "--telnet")

class F20_TestCase(F14_TestCase):
    def runTest(self):
        F14_TestCase.runTest(self)

        # remove service
        self.assert_parse("firewall --remove-service=mdns",
                          "firewall --enabled --remove-service=mdns\n")  # remove only
        # service & remove service at once
        self.assert_parse("firewall --service=ssh --remove-service=mdns",
                          "firewall --enabled --service=ssh --remove-service=mdns\n")
        # with alternative service notation
        self.assert_parse("firewall --ssh --smtp --ftp --remove-service=mdns",
                          "firewall --enabled --service=ssh,smtp,ftp --remove-service=mdns\n")
        # multiple remove & remove ssh
        self.assert_parse("firewall --service=mdns --remove-service=dhcpv6-client --remove-service=ssh",
                          "firewall --enabled --service=mdns --remove-service=dhcpv6-client,ssh\n")

class F28_TestCase(F20_TestCase):
    def runTest(self):
        F20_TestCase.runTest(self)

        # use-system-defaults
        self.assert_parse("firewall --use-system-defaults", "firewall --use-system-defaults\n")
        self.assert_parse("firewall --enabled --service=ssh --use-system-defaults", "firewall --use-system-defaults\n")

if __name__ == "__main__":
    unittest.main()



import six
import sys
import unittest
import tempfile
import os

from tests.baseclass import CommandTest, loadModules

from pykickstart.version import *           # pylint: disable=wildcard-import
from pykickstart.errors import KickstartVersionError

def getClassName(cls):
    return cls().__class__.__name__

class StringToVersion_TestCase(CommandTest):
    def runTest(self):

        # fail - no version specified
        self.assertRaises(KickstartVersionError, stringToVersion, "RHEL")
        self.assertRaises(KickstartVersionError, stringToVersion, "Red Hat Enterprise Linux")
        self.assertRaises(KickstartVersionError, stringToVersion, "Fedora")
        self.assertRaises(KickstartVersionError, stringToVersion, "F")
        self.assertRaises(KickstartVersionError, stringToVersion, "FC")

        # fail - too old
        self.assertRaises(KickstartVersionError, stringToVersion, "Fedora Core 1")
        self.assertRaises(KickstartVersionError, stringToVersion, "Fedora Core 2")

        # fail - incorrect syntax
        self.assertRaises(KickstartVersionError, stringToVersion, "FC7")
        self.assertRaises(KickstartVersionError, stringToVersion, "FC8")
        self.assertRaises(KickstartVersionError, stringToVersion, "FC9")
        self.assertRaises(KickstartVersionError, stringToVersion, "FC10")
        self.assertRaises(KickstartVersionError, stringToVersion, "FC11")
        self.assertRaises(KickstartVersionError, stringToVersion, "F1111")
        self.assertRaises(KickstartVersionError, stringToVersion, "F 11")

        # fail - doesn't exist
        self.assertRaises(KickstartVersionError, stringToVersion, "Red hat Enterprise Linux 1")

        # pass - FC3
        self.assertEqual(stringToVersion("Fedora Core 3"), FC3)
        self.assertEqual(stringToVersion("FC3"), FC3)
        # pass - FC4
        self.assertEqual(stringToVersion("Fedora Core 4"), FC4)
        self.assertEqual(stringToVersion("FC4"), FC4)
        # pass - FC5
        self.assertEqual(stringToVersion("Fedora Core 5"), FC5)
        self.assertEqual(stringToVersion("FC5"), FC5)
        # pass - FC6
        self.assertEqual(stringToVersion("Fedora Core 6"), FC6)
        self.assertEqual(stringToVersion("FC6"), FC6)
        # pass - F7
        self.assertEqual(stringToVersion("Fedora Core 7"), F7)
        self.assertEqual(stringToVersion("Fedora 7"), F7)
        self.assertEqual(stringToVersion("F7"), F7)
        # pass - F8
        self.assertEqual(stringToVersion("Fedora Core 8"), F8)
        self.assertEqual(stringToVersion("Fedora 8"), F8)
        self.assertEqual(stringToVersion("F8"), F8)
        # pass - F9
        self.assertEqual(stringToVersion("Fedora Core 9"), F9)
        self.assertEqual(stringToVersion("Fedora 9"), F9)
        self.assertEqual(stringToVersion("F9"), F9)
        # pass - F10
        self.assertEqual(stringToVersion("Fedora Core 10"), F10)
        self.assertEqual(stringToVersion("Fedora 10"), F10)
        self.assertEqual(stringToVersion("F10"), F10)
        # pass - F11
        self.assertEqual(stringToVersion("Fedora 11"), F11)
        self.assertEqual(stringToVersion("F11"), F11)
        # pass - F12
        self.assertEqual(stringToVersion("Fedora 12"), F12)
        self.assertEqual(stringToVersion("F12"), F12)
        # pass - F13
        self.assertEqual(stringToVersion("Fedora 13"), F13)
        self.assertEqual(stringToVersion("F13"), F13)
        # pass - F14
        self.assertEqual(stringToVersion("Fedora 14"), F14)
        self.assertEqual(stringToVersion("F14"), F14)
        # pass - F15
        self.assertEqual(stringToVersion("Fedora 15"), F15)
        self.assertEqual(stringToVersion("F15"), F15)
        # pass - F16
        self.assertEqual(stringToVersion("Fedora 16"), F16)
        self.assertEqual(stringToVersion("F16"), F16)
        # pass - F17
        self.assertEqual(stringToVersion("Fedora 17"), F17)
        self.assertEqual(stringToVersion("F17"), F17)
        # pass - F18
        self.assertEqual(stringToVersion("Fedora 18"), F18)
        self.assertEqual(stringToVersion("F18"), F18)
        # pass - F19
        self.assertEqual(stringToVersion("Fedora 19"), F19)
        self.assertEqual(stringToVersion("F19"), F19)
        # pass - F20
        self.assertEqual(stringToVersion("Fedora 20"), F20)
        self.assertEqual(stringToVersion("F20"), F20)
        # pass - F21
        self.assertEqual(stringToVersion("Fedora 21"), F21)
        self.assertEqual(stringToVersion("F21"), F21)
        # pass - F22
        self.assertEqual(stringToVersion("Fedora 22"), F22)
        self.assertEqual(stringToVersion("F22"), F22)
        # pass - F23
        self.assertEqual(stringToVersion("Fedora 23"), F23)
        self.assertEqual(stringToVersion("F23"), F23)
        # pass - F24
        self.assertEqual(stringToVersion("Fedora 24"), F24)
        self.assertEqual(stringToVersion("F24"), F24)
        # pass - F25
        self.assertEqual(stringToVersion("Fedora 25"), F25)
        self.assertEqual(stringToVersion("F25"), F25)
        # pass - F26
        self.assertEqual(stringToVersion("Fedora 26"), F26)
        self.assertEqual(stringToVersion("F26"), F26)
        # pass - F27
        self.assertEqual(stringToVersion("Fedora 27"), F27)
        self.assertEqual(stringToVersion("F27"), F27)
        # pass - F28
        self.assertEqual(stringToVersion("Fedora 28"), F28)
        self.assertEqual(stringToVersion("F28"), F28)
        # pass - F29
        self.assertEqual(stringToVersion("Fedora 29"), F29)
        self.assertEqual(stringToVersion("F29"), F29)
        # pass - F30
        self.assertEqual(stringToVersion("Fedora 30"), F30)
        self.assertEqual(stringToVersion("F30"), F30)
        # pass - F31
        self.assertEqual(stringToVersion("Fedora 31"), F31)
        self.assertEqual(stringToVersion("F31"), F31)
        # pass - F32
        self.assertEqual(stringToVersion("Fedora 32"), F32)
        self.assertEqual(stringToVersion("F32"), F32)
        # pass - F33
        self.assertEqual(stringToVersion("Fedora 33"), F33)
        self.assertEqual(stringToVersion("F33"), F33)
        # pass - F34
        self.assertEqual(stringToVersion("Fedora 34"), F34)
        self.assertEqual(stringToVersion("F34"), F34)

        # pass - RHEL3
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux 3"), RHEL3)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux AS 3"), RHEL3)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux ES 3"), RHEL3)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux WS 3"), RHEL3)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux Desktop 3"), RHEL3)
        self.assertEqual(stringToVersion("RHEL3"), RHEL3)

        # pass - RHEL4
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux 4"), RHEL4)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux AS 4"), RHEL4)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux ES 4"), RHEL4)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux WS 4"), RHEL4)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux Desktop 4"), RHEL4)
        self.assertEqual(stringToVersion("RHEL4"), RHEL4)

        # pass - RHEL5
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux 5"), RHEL5)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux Client 5"), RHEL5)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux Server 5"), RHEL5)
        for MINOR in range(1, 10):
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux 5.%s" % MINOR), RHEL5)
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux Client 5.%s" % MINOR), RHEL5)
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux Server 5.%s" % MINOR), RHEL5)
        self.assertEqual(stringToVersion("RHEL5"), RHEL5)

        # pass - RHEL6
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux 6"), RHEL6)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux Client 6"), RHEL6)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux Server 6"), RHEL6)
        for MINOR in range(1, 10):
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux 6.%s" % MINOR), RHEL6)
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux Client 6.%s" % MINOR), RHEL6)
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux Server 6.%s" % MINOR), RHEL6)
        self.assertEqual(stringToVersion("RHEL6"), RHEL6)

        # pass - RHEL7
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux 7"), RHEL7)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux Client 7"), RHEL7)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux Server 7"), RHEL7)
        for MINOR in range(1, 10):
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux 7.%s" % MINOR), RHEL7)
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux Client 7.%s" % MINOR), RHEL7)
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux Server 7.%s" % MINOR), RHEL7)
        self.assertEqual(stringToVersion("RHEL7"), RHEL7)

        # pass - RHEL8
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux 8"), RHEL8)
        for MINOR in range(1,10):
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux 8.%s" % MINOR), RHEL8)
        self.assertEqual(stringToVersion("RHEL8"), RHEL8)

        # pass - RHEL9
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux 9"), RHEL9)
        for MINOR in range(1,10):
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux 9.%s" % MINOR), RHEL9)
        self.assertEqual(stringToVersion("RHEL9"), RHEL9)


class VersionToString_TestCase(CommandTest):
    def runTest(self):

        # Make sure DEVEL is the highest version, but RHEL versions aren't
        # counted as devel.
        highest = 0
        for (ver_str, ver_num) in list(versionMap.items()):
            if ver_str.startswith("RHEL"):
                continue

            highest = max(ver_num, highest)
        self.assertEqual(highest, DEVEL)

        # FC series
        self.assertEqual(versionToString(FC3), "FC3")
        self.assertEqual(versionToString(FC4), "FC4")
        self.assertEqual(versionToString(FC5), "FC5")
        self.assertEqual(versionToString(FC6), "FC6")
        # F series
        self.assertEqual(versionToString(F7), "F7")
        self.assertEqual(versionToString(F8), "F8")
        self.assertEqual(versionToString(F9), "F9")
        self.assertEqual(versionToString(F10), "F10")
        self.assertEqual(versionToString(F10, skipDevel=True), "F10")
        self.assertEqual(versionToString(F10, skipDevel=False), "F10")
        self.assertEqual(versionToString(F11, skipDevel=True), "F11")
        self.assertEqual(versionToString(F11, skipDevel=False), "F11")
        self.assertEqual(versionToString(F12, skipDevel=True), "F12")
        self.assertEqual(versionToString(F13, skipDevel=True), "F13")
        self.assertEqual(versionToString(F14, skipDevel=True), "F14")
        self.assertEqual(versionToString(F15, skipDevel=True), "F15")
        self.assertEqual(versionToString(F16, skipDevel=True), "F16")
        self.assertEqual(versionToString(F17, skipDevel=True), "F17")
        self.assertEqual(versionToString(F18, skipDevel=True), "F18")
        self.assertEqual(versionToString(F19, skipDevel=True), "F19")
        self.assertEqual(versionToString(F20, skipDevel=True), "F20")
        self.assertEqual(versionToString(F21, skipDevel=True), "F21")
        self.assertEqual(versionToString(F22, skipDevel=True), "F22")
        self.assertEqual(versionToString(F23, skipDevel=True), "F23")
        self.assertEqual(versionToString(F24, skipDevel=True), "F24")
        self.assertEqual(versionToString(F25, skipDevel=True), "F25")
        self.assertEqual(versionToString(F26, skipDevel=True), "F26")
        self.assertEqual(versionToString(F27, skipDevel=True), "F27")
        self.assertEqual(versionToString(F28, skipDevel=True), "F28")
        self.assertEqual(versionToString(F29, skipDevel=True), "F29")
        self.assertEqual(versionToString(F30, skipDevel=True), "F30")
        self.assertEqual(versionToString(F31, skipDevel=True), "F31")
        self.assertEqual(versionToString(F32, skipDevel=True), "F32")
        self.assertEqual(versionToString(F33, skipDevel=True), "F33")
        self.assertEqual(versionToString(F34, skipDevel=True), "F34")
        self.assertEqual(versionToString(F34, skipDevel=False), "DEVEL")
        # RHEL series
        self.assertEqual(versionToString(RHEL3), "RHEL3")
        self.assertEqual(versionToString(RHEL4), "RHEL4")
        self.assertEqual(versionToString(RHEL5), "RHEL5")
        self.assertEqual(versionToString(RHEL6), "RHEL6")
        self.assertEqual(versionToString(RHEL7), "RHEL7")
        self.assertEqual(versionToString(RHEL8), "RHEL8")
        self.assertEqual(versionToString(RHEL9), "RHEL9")

        # fail
        self.assertRaises(KickstartVersionError, versionToString, 47)

class returnClassForVersion_TestCase(CommandTest):
    def runTest(self):

        # Test that everything in version.versionMap has a handler, except
        # for DEVEL.
        for (name, vers) in list(versionMap.items()):
            if name == "DEVEL":
                continue

            self.assertEqual(returnClassForVersion(vers).version, vers)

        # test that unknown version will raise an exception
        import pykickstart.version as ver
        orig_versionToString = ver.versionToString
        with self.assertRaises(KickstartVersionError):
            def fake_versionToString(version, skipDevel=False):
                return "-1"

            ver.versionToString = fake_versionToString
            ver.returnClassForVersion(-1)
        ver.versionToString = orig_versionToString

        # Load the handlers
        _path = os.path.join(os.path.dirname(__file__), "..", "pykickstart", "handlers")
        _path = os.path.abspath(_path)
        for module in loadModules(_path, cls_pattern="Handler", skip_list=["control"]):
            if module.__name__.endswith("Handler") and module.__name__ not in ["BaseHandler"]:
                # What is the version of the handler?
                vers = module.__name__.replace("Handler", "")
                self.assertTrue(vers in versionMap)
                # Ensure that returnClassForVersion returns what we expect
                self.assertEqual(getClassName(returnClassForVersion(versionMap[vers])), getClassName(module))

class versionFromFile_TestCase(CommandTest):
    def runTest(self):

        def write_ks_cfg(buf):
            (fd, name) = tempfile.mkstemp(prefix="ks-", suffix=".cfg", dir="/tmp", text=True)
            if six.PY3:
                buf = buf.encode(sys.getdefaultencoding())
            os.write(fd, buf)
            os.close(fd)
            return name

        # no version specified
        ks_cfg = '''
# This is a sample kickstart file
rootpw testing123
cdrom
'''
        ks_cfg = write_ks_cfg(ks_cfg)
        self.assertEqual(versionFromFile(ks_cfg), DEVEL)
        os.unlink(ks_cfg)

        # proper format ... DEVEL
        ks_cfg = '''
# This is a sample kickstart file
#version=DEVEL
rootpw testing123
cdrom
'''
        ks_cfg = write_ks_cfg(ks_cfg)
        self.assertEqual(versionFromFile(ks_cfg), DEVEL)
        os.unlink(ks_cfg)

        # proper format ... RHEL3
        ks_cfg = '''
# This is a sample kickstart file
#version=RHEL3
rootpw testing123
cdrom
'''
        ks_cfg = write_ks_cfg(ks_cfg)
        self.assertEqual(versionFromFile(ks_cfg), RHEL3)
        os.unlink(ks_cfg)

        # improper format ... fallback to DEVEL
        ks_cfg = '''
# This is a sample kickstart file
# version: FC3
rootpw testing123
cdrom
'''
        ks_cfg = write_ks_cfg(ks_cfg)
        self.assertEqual(versionFromFile(ks_cfg), DEVEL)
        os.unlink(ks_cfg)

        # unknown version specified ... raise exception
        ks_cfg = '''
# This is a sample kickstart file
#version=RHEL5000
rootpw testing123
cdrom
'''
        ks_cfg = write_ks_cfg(ks_cfg)
        self.assertRaises(KickstartVersionError, versionFromFile, ks_cfg)
        os.unlink(ks_cfg)

class FailedImpImport_TestCase(CommandTest):
    def runTest(self):
        try:
            # will force another import
            del sys.modules['pykickstart.version']
            del sys.modules['importlib']

            import pykickstart.version as ver
            from pykickstart.handlers.f23 import F23Handler

            cls = ver.returnClassForVersion(ver.F23)

            # assert the names; b/c of how the importlib.import_module() works
            # asserting both classes being equal fails
            self.assertEqual(cls.__name__, F23Handler.__name__)
        finally:
            # force import to reload these modules
            del sys.modules['pykickstart.version']
            del sys.modules['importlib']

if __name__ == "__main__":
    unittest.main()



#
# Martin Kolman <mkolman@redhat.com>
#
# Copyright 2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#


import unittest
from tests.baseclass import CommandTest


class RHEL8_TestCase(CommandTest):
    def runTest(self):
        # basic parsing
        self.assert_parse('syspurpose --role="foo" --sla="bar" --usage="baz" --addon="a" --addon="b" --addon="c"')
        self.assert_parse('syspurpose --role="foo"')
        self.assert_parse('syspurpose --sla="bar"')
        self.assert_parse('syspurpose --usage="baz"')
        self.assert_parse('syspurpose --addon="a" --addon="b" --addon="c"')
        # just syspurpose without options is likely valid even though useless
        self.assert_parse('syspurpose')

        # multi word names
        self.assert_parse('syspurpose --role="foo a" --sla="bar b c" --usage="baz foo" --addon="a b" --addon="c d" --addon="e"')

        # no positional arguments are accepted
        self.assert_parse_error('syspurpose foo')
        self.assert_parse_error('syspurpose foo --role=foo --sla=bar --usage=baz')

        # unknown options are an error
        self.assert_parse_error('syspurpose --role=foo --unknown=stuff')

        # test output kickstart generation
        self.assert_parse("syspurpose", "")
        self.assert_parse('syspurpose --role="foo"', 'syspurpose --role="foo"\n')
        self.assert_parse('syspurpose --sla="bar"', 'syspurpose --sla="bar"\n')
        self.assert_parse('syspurpose --usage="baz"', 'syspurpose --usage="baz"\n')
        self.assert_parse('syspurpose --role="foo" --sla="bar" --usage="baz"\n',
                          'syspurpose --role="foo" --sla="bar" --usage="baz"\n')
        self.assert_parse('syspurpose --addon="a" --addon="b" --addon="c"', 'syspurpose --addon="a" --addon="b" --addon="c"\n')
        # multi word handling
        self.assert_parse('syspurpose --role="foo a" --sla="bar b" --usage="baz c" --addon="a b" --addon="c d" --addon="e"',
                          'syspurpose --role="foo a" --sla="bar b" --usage="baz c" --addon="a b" --addon="c d" --addon="e"\n')

if __name__ == "__main__":
    unittest.main()



#
# Martin Kolman <mkolman@redhat.com>
#
# Copyright 2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#


import unittest
from tests.baseclass import CommandTest


class F29_TestCase(CommandTest):
    def runTest(self):
        # basic parsing
        self.assert_parse('module --name=nodejs --stream=6', 'module --name=nodejs --stream=6\n')
        self.assert_parse('module --name=foo --stream=1337', 'module --name=foo --stream=1337\n')
        # non integer stream ids should also be fine
        self.assert_parse('module --name=bar --stream=baz', 'module --name=bar --stream=baz\n')

        # --stream is optional
        self.assert_parse_error('module')
        self.assert_parse('module --name=foo', 'module --name=foo\n')

        # but name needs to be always present
        self.assert_parse_error('module --stream=bar')

        # the values must not be empty
        self.assert_parse_error('module foo --name=bar --stream=')
        self.assert_parse_error('module foo --name= --stream=baz')
        self.assert_parse_error('module foo --name= --stream=')
        self.assert_parse_error('module foo --name --stream')
        self.assert_parse_error('module foo --name')

        # the module command does not take any absolute arguments
        self.assert_parse_error('module foo')
        self.assert_parse_error('module foo --name=bar')
        self.assert_parse_error('module foo --name=bar --stream=baz')

        # unknown options are an error
        self.assert_parse_error('module --name=bar --stream=baz --uknown=stuff')
        self.assert_parse_error('module --name=bar --uknown=stuff')


class RHEL8_TestCase(F29_TestCase):
    def runTest(self):
        # run F29 test case.
        F29_TestCase.runTest(self)

        # parse --disable
        self.assert_parse('module --name=nodejs --stream=6 --disable', 'module --name=nodejs --stream=6 --disable\n')
        self.assert_parse('module --name=foo --stream=1337 --disable', 'module --name=foo --stream=1337 --disable\n')

        # no assignment to the --disable option
        self.assert_parse_error('module --name=nodejs --stream=6 --disable=bar')
        self.assert_parse_error('module --name=foo --stream=1337 --disable=bar')


class F31_TestCase(RHEL8_TestCase):
    def runTest(self):
        # run RHEL8 test case.
        RHEL8_TestCase.runTest(self)


if __name__ == "__main__":
    unittest.main()



#
# Brian C. Lane <bcl@redhat.com>
#
# Copyright 2012 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import unittest
from tests.baseclass import CommandTest
from pykickstart.commands.reboot import FC3_Reboot, FC6_Reboot, F18_Reboot, F23_Reboot
from pykickstart.constants import KS_REBOOT, KS_SHUTDOWN, KS_WAIT

class Reboot_TestCase(unittest.TestCase):
    def runTest(self):
        for reboot_class in [FC3_Reboot, F18_Reboot]:
            cmd = reboot_class()

            for action in [-1, 9999]:
                cmd.action = action
                self.assertEqual(cmd.__str__(), "")

            cmd.action = KS_REBOOT
            self.assertEqual(cmd.__str__(), "# Reboot after installation\nreboot\n")

            cmd.action = KS_SHUTDOWN
            self.assertEqual(cmd.__str__(), "# Shutdown after installation\nshutdown\n")

            if isinstance(cmd, F18_Reboot):
                cmd.action = KS_WAIT
                self.assertEqual(cmd.__str__(), "# Halt after installation\nhalt\n")

            for currentCmd in ['aaaaa', 'zzzzz']:
                cmd.currentCmd = currentCmd
                cmd.action = None
                cmd.parse([])
                self.assertEqual(cmd.action, None)


class FC3_TestCase(CommandTest):
    command = "reboot"

    def runTest(self):
        # pass
        cmd = self.assert_parse("reboot")
        self.assertEqual(cmd.action, KS_REBOOT)
        self.assertEqual(str(cmd), "# Reboot after installation\nreboot\n")
        cmd = self.assert_parse("shutdown")
        self.assertEqual(cmd.action, KS_SHUTDOWN)
        self.assertEqual(str(cmd), "# Shutdown after installation\nshutdown\n")

        cmd = self.assert_parse("poweroff")
        self.assertEqual(cmd.action, KS_SHUTDOWN)

class FC6_TestCase(FC3_TestCase):
    def runTest(self):
        FC3_TestCase.runTest(self)

        # assert default values
        cmd = FC6_Reboot()
        self.assertFalse(cmd.eject)

        # pass
        cmd = self.assert_parse("reboot --eject")
        self.assertEqual(cmd.action, KS_REBOOT)
        self.assertEqual(cmd.eject, True)
        self.assertEqual(str(cmd), "# Reboot after installation\nreboot --eject\n")

class F18_TestCase(FC6_TestCase):
    def runTest(self):
        FC6_TestCase.runTest(self)

        # pass
        cmd = self.assert_parse("halt")
        self.assertEqual(cmd.action, KS_WAIT)
        self.assertEqual(str(cmd), "# Halt after installation\nhalt\n")
        cmd = self.assert_parse("halt --eject")
        self.assertEqual(cmd.eject, True)
        self.assertEqual(str(cmd), "# Halt after installation\nhalt --eject\n")

        parser = cmd._getParser()
        self.assertTrue(parser.description.find('versionchanged:: Fedora18') > -1)

class F23_TestCase(F18_TestCase):
    def runTest(self):
        F18_TestCase.runTest(self)

        # assert default values
        cmd = F23_Reboot()
        self.assertFalse(cmd.kexec)

        # pass
        cmd = self.assert_parse("reboot --kexec")
        self.assertEqual(cmd.kexec, True)
        self.assertEqual(str(cmd), "# Reboot after installation\nreboot --kexec\n")
        cmd = self.assert_parse("reboot --eject --kexec")
        self.assertEqual(cmd.kexec, True)
        self.assertEqual(cmd.eject, True)
        self.assertEqual(str(cmd), "# Reboot after installation\nreboot --eject --kexec\n")


if __name__ == "__main__":
    unittest.main()



#
# Colin Walters <walters@redhat.com>
#
# Copyright 2014 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from tests.baseclass import CommandTest
from pykickstart.commands.ostreesetup import F21_OSTreeSetup

class OSTreeSetup_TestCase(unittest.TestCase):
    def runTest(self):
        cmd = F21_OSTreeSetup()
        self.assertEqual(cmd.nogpg, False)

        # no remote, should equal osname
        cmd.parse('--osname=fedora-atomic --url=http://example.com/repo --ref=fedora-atomic/sometest/base/core'.split(' '))
        self.assertEqual(cmd.remote, cmd.osname)

        # test if arguments are required
        op = cmd._getParser()
        for action in op._actions:
            for a in ['--osname', '--url', '--ref']:
                if a in action.option_strings:
                    self.assertEqual(action.required, True)

class F21_TestCase(CommandTest):
    command = "ostreesetup"

    def runTest(self):
        # pass
        self.assert_parse("ostreesetup --osname=fedora-atomic --url=http://example.com/repo --ref=fedora-atomic/sometest/base/core")
        self.assert_parse("ostreesetup --osname=local-atomic --url=file:///home/ostree --ref=fedora-atomic/sometest/base/core")
        cmdstr = "ostreesetup --osname=\"fedora-atomic\" --remote=\"fedora-atomic\" --url=\"http://example.com/repo\" --ref=\"fedora-atomic/sometest/base/core\" --nogpg"
        self.assert_parse(cmdstr, cmdstr + '\n')

        # fail - we have required arguments
        self.assert_parse_error("ostreesetup")
        self.assert_parse_error("ostreesetup --os=fedora-atomic")
        self.assert_parse_error("ostreesetup --os=fedora-atomic --url=http://example.com/repo")
        self.assert_parse_error("ostreesetup --bacon=tasty")

        # fail - wrong protocol for repo
        self.assert_parse_error("ostreesetup --osname=fedora-atomic --url=ftp://example.com/repo --ref=fedora-atomic/sometest/base/core")

        # extra test coverage
        cmd = self.handler().commands[self.command]
        cmd.osname = ""
        cmd.remote = ""
        cmd.url = ""
        cmd.ref = ""
        cmd.nogpg = False
        self.assertEqual(cmd._getArgsAsStr(), "")

class RHEL7_TestCase(F21_TestCase):
    def runTest(self):
        F21_TestCase.runTest(self)

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from textwrap import dedent
from pykickstart.version import versionToLongString, FC3, F11, F20, F29
from pykickstart.base import DeprecatedCommand, KickstartCommand, RemovedCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser

from pykickstart.i18n import _

class FC3_Upgrade(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.upgrade = kwargs.get("upgrade", None)
        self.op = self._getParser()

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.upgrade is None:
            return retval

        if self.upgrade:
            retval += "# Upgrade existing installation\nupgrade\n"
        else:
            retval += "# Install OS instead of upgrade\ninstall\n"

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="install|upgrade", description="""
                            Install a fresh system or upgrade an existing system.
                            Install is the default mode. For installation, you must
                            specify the type of installation from one of
                            cdrom, harddrive, nfs, or url (for ftp or http installations).
                            The install command and the installation method command
                            must be on separate lines.""",
                            version=FC3)
        return op

    def parse(self, args):
        self.op.parse_args(args=args, lineno=self.lineno)

        if self.currentCmd == "upgrade":
            self.upgrade = True
        else:
            self.upgrade = False

        return self

class F11_Upgrade(FC3_Upgrade):
    removedKeywords = FC3_Upgrade.removedKeywords
    removedAttrs = FC3_Upgrade.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        FC3_Upgrade.__init__(self, writePriority, *args, **kwargs)

        self.op = self._getParser()
        self.root_device = kwargs.get("root_device", None)

    def __str__(self):
        if self.upgrade and (self.root_device is not None):
            retval = KickstartCommand.__str__(self)
            retval += "# Upgrade existing installation\nupgrade --root-device=%s\n" % self.root_device
        else:
            retval = FC3_Upgrade.__str__(self)

        return retval

    def _getParser(self):
        op = FC3_Upgrade._getParser(self)
        op.add_argument("--root-device", dest="root_device", version=F11,
                        help="""
                        On a system with multiple installs, this option specifies which
                        filesystem holds the installation to be upgraded. This can be
                        specified by device name, UUID=, or LABEL= just like the harddrive
                        command may be.""")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)

        if ns.root_device == "":
            raise KickstartParseError(_("Kickstart command %(command)s does not accept empty parameter %(parameter)s") % {"command": "upgrade", "parameter": "--root-device"}, lineno=self.lineno)
        else:
            self.root_device = ns.root_device

        if self.currentCmd == "upgrade":
            self.upgrade = True
        else:
            self.upgrade = False

        return self

class F20_Upgrade(DeprecatedCommand, F11_Upgrade):
    def __init__(self):  # pylint: disable=super-init-not-called
        DeprecatedCommand.__init__(self)

    def _getParser(self):
        op = F11_Upgrade._getParser(self)
        op.description += dedent("""

                        .. deprecated:: %s

                        Starting with F18, upgrades are no longer supported in
                        anaconda and should be done with FedUp, the Fedora update
                        tool. Starting with F21, the DNF system-upgrade plugin is
                        recommended instead.  Therefore, the upgrade command
                        essentially does nothing.""" % versionToLongString(F20))
        return op

class F29_Upgrade(RemovedCommand, F20_Upgrade):
    def _getParser(self):
        op = F20_Upgrade._getParser(self)
        op.description += "\n\n.. versionremoved:: %s" % versionToLongString(F29)
        return op



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3, FC5, F7, versionToLongString
from pykickstart.base import DeprecatedCommand, KickstartCommand, RemovedCommand
from pykickstart.errors import KickstartParseError
from pykickstart.i18n import _
from pykickstart.options import KSOptionParser

class FC3_LangSupport(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.deflang = kwargs.get("deflang", "")
        self.supported = kwargs.get("supported", [])

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.deflang:
            retval += "langsupport --default=%s" % self.deflang

            if self.supported:
                retval += " %s" % " ".join(self.supported)

        return retval + "\n"

    def _getParser(self):
        op = KSOptionParser(prog="langsupport", description="""
            Install the support packages for the given locales.""", version=FC3)
        op.add_argument("--default", dest="deflang", default="en_US.UTF-8",
                        version=FC3, help="Default locale")
        return op

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)
        if any(arg for arg in extra if arg.startswith("-")):
            mapping = {"command": "langsupport", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)

        self.set_to_self(ns)
        self.supported = extra
        return self

class FC5_LangSupport(DeprecatedCommand, FC3_LangSupport):
    def __init__(self):  # pylint: disable=super-init-not-called
        DeprecatedCommand.__init__(self)

    def _getParser(self):
        op = FC3_LangSupport._getParser(self)
        op.description += "\n\n.. deprecated:: %s" % versionToLongString(FC5)
        return op

class F7_LangSupport(RemovedCommand, FC5_LangSupport):
    def _getParser(self):
        op = FC5_LangSupport._getParser(self)
        op.description += "\n\n.. versionremoved:: %s" % versionToLongString(F7)
        return op



#
# Copyright (C) 2013  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): Vratislav Podzimek <vpodzime@redhat.com>
#

import unittest
from tests.baseclass import CommandTest
from pykickstart.commands.eula import F20_Eula

class Eula_TestCase(unittest.TestCase):
    def runTest(self):
        cmd = F20_Eula()
        self.assertEqual(cmd.agreed, False)

class F20_TestCase(CommandTest):
    command = "eula"

    def runTest(self):
        # pass
        self.assert_parse("eula --agreed", "eula --agreed\n")
        self.assert_parse("eula --agree", "eula --agreed\n")
        self.assert_parse("eula --accepted", "eula --agreed\n")
        self.assert_parse("eula --accept", "eula --agreed\n")

        # fail
        self.assert_parse_error("eula")
        self.assert_parse_error("eula arg1")

        # extra
        eula = self.handler().commands["eula"]
        eula.agreed = False
        self.assertTrue(eula._getArgsAsStr() == "")

if __name__ == "__main__":
    unittest.main()



#
# David Lehman <dlehman@redhat.com>
#
# Copyright 2011 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from tests.baseclass import CommandTest, CommandSequenceTest
from pykickstart.commands.btrfs import F17_BTRFSData, F23_BTRFSData
from pykickstart.errors import KickstartParseError, KickstartParseWarning
from pykickstart.version import F17

class BTRFS_TestCase(unittest.TestCase):
    def runTest(self):
        data1 = F17_BTRFSData()
        data2 = F17_BTRFSData()

        # test default attribute values
        self.assertEqual(data1.format, True)
        self.assertEqual(data1.preexist, False)
        self.assertEqual(data1.subvol, False)

        # test that new objects are always equal
        self.assertEqual(data1, data2)
        self.assertNotEqual(data1, None)

        # test for objects difference
        for atr in ['mountpoint']:
            setattr(data1, atr, '')
            setattr(data2, atr, '/test')
            # objects that differ in only one attribute
            # are not equal
            self.assertNotEqual(data1, data2)
            self.assertNotEqual(data2, data1)
            setattr(data1, atr, '')
            setattr(data2, atr, '')

        # test for attribute values based on prefered
        # parameter names
        for attr, attr_alt in [('data', 'dataLevel'),
                               ('metadata', 'metaDataLevel')]:
            for (v1, v2) in [(1, None), (None, 2), (1, 2)]:
                kwargs = { attr: v1, attr_alt: v2 }
                data = F17_BTRFSData(**kwargs)
                self.assertEqual(getattr(data, attr_alt), v1 or v2)

        for attr, attr_alt in [('mkfsoptions', 'mkfsopts')]:
            for (v1, v2) in [(1, ''), ('', 2), (1, 2)]:
                kwargs = { attr: v1, attr_alt: v2 }
                data = F23_BTRFSData(**kwargs)
                self.assertEqual(getattr(data, attr_alt), v1 or v2)


class F17_TestCase(CommandTest):
    command = "btrfs"

    def __init__(self, *kargs, **kwargs):
        CommandTest.__init__(self, *kargs, **kwargs)
        self.validLevels = ["raid0", "raid1", "raid10", "single"]

    def runTest(self):
        # valid levels
        pre = "btrfs /"
        post = "btrfs.01"
        for data in self.validLevels:
            self.assert_parse("%s --data=%s %s" % (pre, data, post))
            for meta in self.validLevels:
                self.assert_parse("%s --data=%s --metadata=%s %s" % (pre, data,
                                                                     meta, post))
                self.assert_parse("%s --metadata=%s %s" % (pre, meta, post))

        # no mountpoint or options ... just partitions
        self.assert_parse("btrfs none part.01 part.01",
                          "btrfs none part.01 part.01\n")

        # useexisting
        self.assert_parse("btrfs /foo --data=1 --useexisting LABEL=foo",
                          "btrfs /foo --noformat --useexisting --data=raid1 LABEL=foo\n")
        self.assert_parse("btrfs /foo --data=RAID1 --useexisting LABEL=foo",
                          "btrfs /foo --noformat --useexisting --data=raid1 LABEL=foo\n")
        self.assert_parse("btrfs /foo --data=raid1 --useexisting LABEL=foo",
                          "btrfs /foo --noformat --useexisting --data=raid1 LABEL=foo\n")
        self.assert_parse("btrfs /foo --data=1 --useexisting LABEL=foo",
                          "btrfs /foo --noformat --useexisting --data=raid1 LABEL=foo\n")

        # noformat
        self.assert_parse("btrfs /foo --data=1 --noformat --useexisting LABEL=foo",
                          "btrfs /foo --noformat --useexisting --data=raid1 LABEL=foo\n")
        self.assert_parse("btrfs /foo --data=RAID1 --noformat --useexisting LABEL=foo",
                          "btrfs /foo --noformat --useexisting --data=raid1 LABEL=foo\n")

        # fail
        # no mountpoint or options
        self.assert_parse_error("btrfs")

        # no options
        self.assert_parse_error("btrfs /")

        # invalid argument
        self.assert_parse_error("btrfs / --bogus-option")

        # No members
        self.assert_parse_error("btrfs / --data=0 --label=root")

        # subvol with no name
        self.assert_parse_error("btrfs / --subvol LABEL=test")

        # subvol with no parent
        self.assert_parse_error("btrfs / --subvol --name=root")

        # bad level
        self.assert_parse_error("btrfs / --data=47 btrfs.01", KickstartParseError, 'Invalid btrfs level: 47')
        self.assert_parse_error("btrfs / --metadata=47 btrfs.01")

        self.assert_parse("btrfs / --subvol --name=root LABEL=test",
                          "btrfs / --subvol --name=root LABEL=test\n")
        self.assert_parse("btrfs / --subvol --name=root test",
                          "btrfs / --subvol --name=root test\n")

        # preexisting
        self.assert_parse("btrfs / --useexisting btrfs.01 btrfs.02",
                          "btrfs / --noformat --useexisting btrfs.01 btrfs.02\n")
        self.assert_parse("btrfs / --useexisting LABEL=test",
                          "btrfs / --noformat --useexisting LABEL=test\n")

        # preexisting subvol with parent specified by label
        self.assert_parse("btrfs /home --subvol --name=home --useexisting LABEL=test",
                          "btrfs /home --noformat --useexisting --subvol --name=home LABEL=test\n")

        # pass
        self.assert_parse("btrfs / --label=ROOT --data=1 part.01 part.02",
                          "btrfs / --label=ROOT --data=raid1 part.01 part.02\n")
        self.assert_parse("btrfs / --data=RAID1 --label=ROOT part.01 part.02",
                          "btrfs / --label=ROOT --data=raid1 part.01 part.02\n")
        self.assert_parse("btrfs / --label=ROOT --metadata=1 part.01 part.02",
                          "btrfs / --label=ROOT --metadata=raid1 part.01 part.02\n")

        # extra test coverage
        btrfs = self.handler().commands["btrfs"]
        btrfs.btrfsList.append(btrfs.parse("btrfs /home --subvol --name=home".split(" ")))

        # excercise F17_BTRFS.__str__() with non-empty btrfsList
        self.assertEqual(btrfs.__str__(), "btrfs btrfs --subvol --name=home /home\n")

        # check for duplicates in the btrfslist when parsing
        with self.assertRaises(KickstartParseWarning):
            btrfs.parse("btrfs /home --subvol --name=home".split(" "))

        # equality
        self.assertNotEqual(self.assert_parse("btrfs / part.01"), None)
        self.assertEqual(self.assert_parse("btrfs / part.01"), self.assert_parse("btrfs / part.01"))
        self.assertEqual(self.assert_parse("btrfs / part.01"), self.assert_parse("btrfs / part.02"))
        self.assertNotEqual(self.assert_parse("btrfs / part.01"), self.assert_parse("btrfs /home part.01"))

class F17_Duplicate_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = F17

    def runTest(self):
        self.assert_parse_error("""
btrfs / --data=1 part.01 part.02
btrfs / --data=1 part.01 part.02""",
            UserWarning, 'A btrfs volume with the mountpoint / has already been defined.')

class F23_TestCase(F17_TestCase):
    def runTest(self):
        F17_TestCase.runTest(self)

        # pass
        self.assert_parse("btrfs / --mkfsoptions=some,thing part.01",
                          "btrfs / --mkfsoptions=\"some,thing\" part.01\n")

        # can't use --mkfsoptions if you're not formatting
        self.assert_parse_error("btrfs / --useexisting --mkfsoptions=whatever")
        self.assert_parse_error("btrfs / --noformat --mkfsoptions=whatever")

        # extra test coverage
        cmd = self.handler().commands[self.command]
        with self.assertRaises(KickstartParseError):
            cmd.parse(["btrfs", "/", "--noformat", "--useexisting", "--mkfsoptions=whatever"])

class RHEL7_TestCase(F23_TestCase):
    def runTest(self):
        F23_TestCase.runTest(self)

if __name__ == "__main__":
    unittest.main()



#
# Hans de Goede <hdegoede@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import F12, F13, F28, RHEL7
from pykickstart.errors import KickstartParseWarning
from pykickstart.base import BaseData, KickstartCommand
from pykickstart.options import KSOptionParser

import warnings
from pykickstart.i18n import _

class F12_FcoeData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.nic = kwargs.get("nic", None)

    def __eq__(self, y):
        if not y:
            return False

        return self.nic == y.nic

    def __ne__(self, y):
        return not self == y

    def _getArgsAsStr(self):
        retval = ""

        if self.nic:
            retval += " --nic=%s" % self.nic

        return retval

    def __str__(self):
        retval = BaseData.__str__(self)
        retval += "fcoe%s\n" % self._getArgsAsStr()
        return retval

class F13_FcoeData(F12_FcoeData):
    removedKeywords = F12_FcoeData.removedKeywords
    removedAttrs = F12_FcoeData.removedAttrs

    def __init__(self, *args, **kwargs):
        F12_FcoeData.__init__(self, *args, **kwargs)
        self.dcb = kwargs.get("dcb", False)

    def _getArgsAsStr(self):
        retval = F12_FcoeData._getArgsAsStr(self)

        if self.dcb:
            retval += " --dcb"

        return retval

class RHEL7_FcoeData(F13_FcoeData):
    removedKeywords = F13_FcoeData.removedKeywords
    removedAttrs = F13_FcoeData.removedAttrs

    def __init__(self, *args, **kwargs):
        F13_FcoeData.__init__(self, *args, **kwargs)
        self.autovlan = kwargs.get("autovlan", False)

    def _getArgsAsStr(self):
        retval = F13_FcoeData._getArgsAsStr(self)

        if self.autovlan:
            retval += " --autovlan"

        return retval

class F28_FcoeData(F13_FcoeData):
    removedKeywords = F13_FcoeData.removedKeywords
    removedAttrs = F13_FcoeData.removedAttrs

    def __init__(self, *args, **kwargs):
        F13_FcoeData.__init__(self, *args, **kwargs)
        self.autovlan = kwargs.get("autovlan", False)

    def _getArgsAsStr(self):
        retval = F13_FcoeData._getArgsAsStr(self)

        if self.autovlan:
            retval += " --autovlan"

        return retval

class RHEL8_FcoeData(F28_FcoeData):
    pass

class RHEL9_FcoeData(F28_FcoeData):
    pass

class F12_Fcoe(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=71, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()
        self.fcoe = kwargs.get("fcoe", [])

    def __str__(self):
        retval = ""
        for fcoe in self.fcoe:
            retval += fcoe.__str__()

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="fcoe", description="""
                            Discover and attach FCoE storage devices accessible via
                            specified network interface
                            """, version=F12)
        op.add_argument("--nic", required=True, version=F12, help="""
                        Name of the network device connected to the FCoE switch""")
        return op

    def parse(self, args):
        zd = self.dataClass()   # pylint: disable=not-callable
        ns = self.op.parse_args(args=args, lineno=self.lineno)

        self.set_to_obj(ns, zd)
        zd.lineno = self.lineno

        # Check for duplicates in the data list.
        if zd in self.dataList():
            warnings.warn(_("A FCOE device with the name %s has already been defined.") % zd.nic, KickstartParseWarning)

        return zd

    def dataList(self):
        return self.fcoe

    @property
    def dataClass(self):
        return self.handler.FcoeData

class F13_Fcoe(F12_Fcoe):
    removedKeywords = F12_Fcoe.removedKeywords
    removedAttrs = F12_Fcoe.removedAttrs

    def _getParser(self):
        op = F12_Fcoe._getParser(self)
        op.add_argument("--dcb", action="store_true", default=False, version=F13, help="""
                        Enable Data Center Bridging awareness in installer. This option
                        should only be enabled for network interfaces that
                        require a host-based DCBX client. Configurations on
                        interfaces that implement a hardware DCBX client should
                        not use it.
                        """)
        return op

class RHEL7_Fcoe(F13_Fcoe):
    removedKeywords = F13_Fcoe.removedKeywords
    removedAttrs = F13_Fcoe.removedAttrs

    def _getParser(self):
        op = F13_Fcoe._getParser(self)
        op.add_argument("--autovlan", action="store_true", default=False, version=RHEL7, help="""
                        Perform automatic VLAN discovery and setup. This option is enabled
                        by default.
                        """)
        return op

class F28_Fcoe(F13_Fcoe):
    removedKeywords = F13_Fcoe.removedKeywords
    removedAttrs = F13_Fcoe.removedAttrs

    def _getParser(self):
        op = F13_Fcoe._getParser(self)
        op.add_argument("--autovlan", action="store_true", default=False, version=F28, help="""
                        Perform automatic VLAN discovery and setup. This option is enabled
                        by default.
                        """)
        return op

class RHEL8_Fcoe(F28_Fcoe):
    pass

class RHEL9_Fcoe(F28_Fcoe):
    pass



import unittest
from tests.baseclass import CommandTest
from pykickstart.errors import KickstartParseError


class F7_TestCase(CommandTest):
    command = "updates"

    def runTest(self):
        # pass
        self.assert_parse("updates", "updates\n")
        self.assert_parse("updates deliciouscheeses", "updates deliciouscheeses\n")

        # fail
        self.assert_parse_error("updates cheese crackers", KickstartParseError, 'Kickstart command updates only takes one argument')
        self.assert_parse_error("updates --bogus-option")


class F34_TestCase(F7_TestCase):

    def runTest(self):
        # don't run F7 tests

        # pass
        self.assert_parse("updates http://path/to/img", "updates http://path/to/img\n")
        self.assert_parse("updates deliciouscheeses", "updates deliciouscheeses\n")

        # fail
        self.assert_parse_error("updates")
        self.assert_parse_error("updates cheese crackers")
        self.assert_parse_error("updates --bogus-option")


if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3, FC6, F9
from pykickstart.base import KickstartCommand
from pykickstart.options import KSOptionParser

class FC3_Vnc(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.enabled = kwargs.get("enabled", False)
        self.password = kwargs.get("password", "")
        self.connect = kwargs.get("connect", "")

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if not self.enabled:
            return retval

        retval += "vnc"

        if self.connect:
            retval += " --connect=%s" % self.connect
        if self.password:
            retval += " --password=%s" % self.password

        return retval + "\n"

    def _getParser(self):
        op = KSOptionParser(prog="vnc", description="""
                            Allows the graphical installation to be viewed
                            remotely via VNC. This method is usually preferred
                            over text mode, as there are some size and language
                            limitations in text installs. With no options, this
                            command will start a VNC server on the machine with
                            no password and will print out the command that
                            needs to be run to connect a remote machine.""",
                            version=FC3)
        op.add_argument("--connect", version=FC3, help="""
                        Connect to a remote host instead of starting VNC server
                        locally.""")
        op.add_argument("--password", version=FC3, help="""
                        Set a password which must be provided to connect to the
                        VNC session. This is optional, but recommended.""")
        return op

    def parse(self, args):
        self.enabled = True
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        return self

class FC6_Vnc(FC3_Vnc):
    removedKeywords = FC3_Vnc.removedKeywords + ["connect"]
    removedAttrs = FC3_Vnc.removedAttrs + ["connect"]

    def __init__(self, writePriority=0, *args, **kwargs):
        FC3_Vnc.__init__(self, writePriority, *args, **kwargs)
        self.deleteRemovedAttrs()

        self.host = kwargs.get("host", "")
        self.port = kwargs.get("port", "")

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if not self.enabled:
            return retval

        retval += "vnc"

        if self.host:
            retval += " --host=%s" % self.host

            if self.port:
                retval += " --port=%s" % self.port
        if self.password:
            retval += " --password=%s" % self.password

        return retval + "\n"

    def _getParser(self):
        op = FC3_Vnc._getParser(self)
        op.add_argument("--connect", dest="_connect", version=FC6,
                        metavar="host[:port]", help="""
                        Added support for host[:port] syntax.""")
        op.add_argument("--host", version=FC6, help="""
                        Instead of starting a VNC server on the install machine,
                        connect to the VNC viewer process listening on the given
                        hostname.""")
        op.add_argument("--port", version=FC6, help="""
                        Provide a port that the remote VNC viewer process is
                        listening on. If not provided, anaconda will use the
                        VNC default.""")
        return op

    def parse(self, args):
        retval = FC3_Vnc.parse(self, args)

        # argparse doesn't give us a way to do something this complicated, so we
        # have to set it on a throwaway value and then go back and fix it up.
        if getattr(retval, "_connect", None):
            cargs = retval._connect.split(":")  # pylint: disable=no-member
            retval.host = cargs[0]              # pylint: disable=attribute-defined-outside-init

            if len(cargs) > 1:
                retval.port = cargs[1]          # pylint: disable=attribute-defined-outside-init

            del(retval._connect)                # pylint: disable=no-member

        return retval

class F9_Vnc(FC6_Vnc):
    removedKeywords = FC6_Vnc.removedKeywords
    removedAttrs = FC6_Vnc.removedAttrs

    def _getParser(self):
        op = FC6_Vnc._getParser(self)
        op.remove_argument("--connect", version=F9)
        return op
